package vn.com.seabank.biz.service;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.seabank.biz.fastoutside.bean.BodyGetITransTrusted;
import com.seabank.biz.fastoutside.bean.BodyGetITransUnTrusted;
import com.seabank.biz.fastoutside.bean.FastOutsideHeader;
import com.seabank.biz.fastoutside.bean.TransactionResponse;
import com.seabank.biz.fastoutside.service.FastOutSideService;
import com.seabank.biz.sto.bean.*;
import com.seabank.biz.sto.service.StoService;
import com.seabank.biz.utility.bean.BankInfo;
import com.swib.seanet.ben.bean.ContactsInfo;
import com.swib.seanet.ben.bean.ContactsResponse;
import com.swib.seanet.ftinside.bean.FTInsideHeader;
import com.swib.seanet.ftinside.bean.FTInsideTransaction;
import com.swib.seanet.ftinside.bean.FTInsideTransactionParam;
import com.swib.seanet.ftinside.service.FTInsideService;
import com.swib.seanet.ftoutside.bean.FTOutsideHeader;
import com.swib.seanet.ftoutside.bean.TransOutsideResponse;
import com.swib.seanet.ftoutside.bean.TransUnTrustedBodyRequest;
import com.swib.seanet.ftoutside.service.FTOutsideService;
import com.swib.seanet.napas.bean.*;
import com.swib.seanet.napas.service.NapasApiClient;
import com.swib.seanet.t24enquiry.bean.BenBank;
import com.swib.seanet.t24enquiry.bean.T24Header;
import com.swib.seanet.t24enquiry.bean.T24ListBankCard;
import com.swib.seanet.t24enquiry.service.T24EnquiryApiClient;
import com.swib.services.config.ApiClientProperties;
import com.swib.services.config.CacheConfiguration;
import com.swib.services.service.SwibCommonService;
import com.swib.services.service.SwibUtilityService;
import com.swib.services.utils.LogUtil;
import com.swib.services.utils.SwibUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import vn.com.seabank.biz.constants.SwibConstants;
import vn.com.seabank.biz.domain.FtAccount;
import vn.com.seabank.biz.domain.UserSessionInfo;
import vn.com.seabank.biz.dto.*;
import vn.com.seabank.biz.exception.CoreApiException;
import vn.com.seabank.biz.utils.JFactory;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.sql.Timestamp;
import java.util.*;
import java.util.stream.Collectors;

import static vn.com.seabank.biz.utils.CommonUtils.MS_FT_PREFIX;
import static vn.com.seabank.biz.utils.ErrorCode.BANK_CODE_NOT_FOUND;

@Service
public class P0405Service {

    private final Logger log = LoggerFactory.getLogger(P0405Service.class);

    private static final String GET_TRANSACTION = "GET_TRANSACTION";
    public static final String GET_TRANS_MAINTAIN = "getTransMaintain";
    private static final String AUTHEN_USER = "USER";
    private static final String FAST_TRANSFER = "FAST-TRANSFER";
    private static final String AUTHEN_TYPE_ENQ_CHECK_ACC = "enqCheckAcc";
    private static final String AUTHEN_TYPE_ENQ_CHECK_CARD = "enqCheckCard";
    private static final String RESULT_OK = "00";
    private static final String FT_CARD_NUMBER = "FT_CARD_NUMBER";
    private static final String FT_ACCOUNT_NUMBER = "FT_ACCOUNT_NUMBER";
    public static final String STO4 = "STO4";
    public static final String GET_ENQUIRY = "GET_ENQUIRY";

    private final ApiClientProperties properties;
    private final SwibUtils swibUtils;
    private final SwibCommonService swibCommonService;
    private final StoService stoService;
    private final SwibUtilityService swibUtilityService;

    public P0405Service(ApiClientProperties properties,
                        SwibUtils swibUtils,
                        SwibCommonService swibCommonService,
                        SwibUtilityService swibUtilityService) {
        this.properties = properties;
//        this.swibEnquiryService = swibEnquiryService;
        this.swibUtils = swibUtils;
        this.swibCommonService = swibCommonService;
        this.swibUtilityService = swibUtilityService;
        this.stoService = new StoService();
    }

    public FTResponseDTO ftInside(HttpServletRequest request, FTInsideDTO dto) throws CoreApiException {
        FTInsideHeader header = createFTInsideHeader(request);

        FTInsideTransactionParam ftParam = new FTInsideTransactionParam();
        BeanUtils.copyProperties(dto, ftParam);

        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        ftParam.setAuthenType("getITransUnTrusted");
        ftParam.setSessionId(userSessionInfo.getSessionId());
        ftParam.setCustomerID(userSessionInfo.getCustomerId());
        ftParam.setTransType(AUTHEN_USER);
        ftParam.setUsername(userSessionInfo.getUsername());
        ftParam.setCheckFeeTrans("YES");
//        ftParam.setTransDesc(SwibUtils.replaceTransDesc(ftParam.getTransDesc()));
        FTInsideService ftInsideService = new FTInsideService(header);
        FTInsideTransaction transaction = ftInsideService.getITransUnTrusted(ftParam);

        FTResponseDTO responseDTO = null;
        if (transaction != null && RESULT_OK.equals(transaction.getResponseCode())) {
            new Thread(() -> saveTransferInside(ftParam)).start();      // save user
            responseDTO = new FTResponseDTO();
            BeanUtils.copyProperties(transaction, responseDTO);
        }
        return responseDTO;
    }

    public FTResponseDTO ftOutside(HttpServletRequest request, FTOutsideDTO dto) {
        FTOutsideHeader header = createFTOHeader(request);
        FTOutsideService ftOutsideService = new FTOutsideService(header);
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        TransUnTrustedBodyRequest ftParam = new TransUnTrustedBodyRequest();
        BeanUtils.copyProperties(dto, ftParam);
        ftParam.setSessionId(userSessionInfo.getSessionId());
        ftParam.setCustomerID(userSessionInfo.getCustomerId());
        ftParam.setAuthenType("getTransUnTrusted");
        ftParam.setTransType(AUTHEN_USER);
        ftParam.setUsername(userSessionInfo.getUsername());
//        ftParam.setTransDesc(SwibUtils.replaceTransDesc(ftParam.getTransDesc()));

        TransOutsideResponse response = ftOutsideService.getTransUnTrusted(header, ftParam);
        FTResponseDTO responseDTO = null;
        if (RESULT_OK.equals(response.getResponseCode())) {
            new Thread(() -> saveTransferOutSide(ftParam)).start();
            responseDTO = new FTResponseDTO();
            BeanUtils.copyProperties(response, responseDTO);
        }

        return responseDTO;
    }

    public FTResponseDTO ftFastOutside(HttpServletRequest request, FTFastOutsideDTO dto) throws CoreApiException {
        FastOutsideHeader fastOutsideHeader = createFastOutSideHeader(request);
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        BodyGetITransUnTrusted ftParam = new BodyGetITransUnTrusted();
        BeanUtils.copyProperties(dto, ftParam);
        ftParam.setCommand(GET_TRANSACTION);
        ftParam.setAuthenType("getITransUnTrustedSwib");
        ftParam.setSessionId(userSessionInfo.getSessionId());
        ftParam.setCustomerID(userSessionInfo.getCustomerId());
        ftParam.setTransType(AUTHEN_USER);
        ftParam.setUsername(userSessionInfo.getUsername());
        ftParam.setIbftChannel(dto.getIbftChannel());
//        ftParam.setTransDesc(SwibUtils.replaceTransDesc(ftParam.getTransDesc()));

        FastOutSideService fastOutSideService = new FastOutSideService(fastOutsideHeader);
        TransactionResponse response = fastOutSideService.getITransUnTrusted(ftParam);
        FTResponseDTO responseDTO = null;
        if (response != null && RESULT_OK.equals(response.getResponseCode())) {
            new Thread(() -> saveTransferFastOutside(ftParam,
                    "CARD".equals(dto.getTransferBy()) ? FT_CARD_NUMBER : FT_ACCOUNT_NUMBER)).start();
            responseDTO = new FTResponseDTO();
            BeanUtils.copyProperties(response, responseDTO);
        }

        return responseDTO;
    }

    public boolean verifyFastOutside(HttpServletRequest request, FTFastOutsideDTO dto) {
        try {
            String userName = swibUtils.getUserName(request);
            BodyGetITransTrusted cache = (BodyGetITransTrusted) CacheConfiguration.getHazelcastInstance()
                    .getMap(FAST_TRANSFER).get(FAST_TRANSFER + userName);

            // remove cache after validate
            CacheConfiguration.getHazelcastInstance().getMap(FAST_TRANSFER).remove(FAST_TRANSFER + userName);

            if (cache.getDebitAccount().equals(dto.getDebitAccount())
                    && cache.getCreditAccount().equals(dto.getCreditAccount())
                    && cache.getTransAmount().equals(dto.getTransAmount())) {
                return true;
            }

        } catch (Exception e) {
            log.error("check information for security error: " + e.getMessage());
            return false;
        }
        return false;
    }

    public EnqCheckCardResponse enqCheckCard(HttpServletRequest request, EnqCheckCardDTO dto)
            throws IOException, CoreApiException {
        log.debug("........Check card......");
        T24ListBankCard bank = null;
        try {
            String sixBin = dto.getBenCard().substring(0, 6);
            List<T24ListBankCard> list = swibUtilityService.getBankCard();
            bank = list.stream().filter(b -> b.getStartBin().contains(sixBin)).findFirst().orElse(null);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        if (bank == null) {
            throw new CoreApiException(BANK_CODE_NOT_FOUND.getCode(), BANK_CODE_NOT_FOUND.getMessage(), MS_FT_PREFIX);
        }

        NapasHeaderRequest napasHeaderRequest = createNapasHeader(request);

        EnqCheckCardBodyRequest bodyRequest = new EnqCheckCardBodyRequest();
        bodyRequest.setAuthenType(AUTHEN_TYPE_ENQ_CHECK_CARD);
        bodyRequest.setBenCard(dto.getBenCard());
        bodyRequest.setSenderAccount(dto.getSenderAccount());
        bodyRequest.setSenderAddress(dto.getSenderAddress());
        bodyRequest.setSenderName(dto.getSenderName());
        bodyRequest.setIbftChannel(dto.getIbftChannel());

        NapasApiClient napasApiClient = new NapasApiClient();
        EnqCheckCardResponse response = napasApiClient.enqCheckCard(napasHeaderRequest, bodyRequest);
        response.setBankCode(bank.getBankCode());
        response.setBankName(bank.getBankName());
        return response;
    }

    public EnqCheckAccResponse enqCheckAcc(HttpServletRequest request, EnqCheckAccountDTO dto)
            throws IOException, CoreApiException {
        log.debug("........Check name Account......");
        NapasHeaderRequest napasHeaderRequest = createNapasHeader(request);
        EnqCheckAccBodyRequest bodyRequest = new EnqCheckAccBodyRequest();
        bodyRequest.setAuthenType(AUTHEN_TYPE_ENQ_CHECK_ACC);
        bodyRequest.setSenderAccount(dto.getSenderAccount());
        bodyRequest.setSenderAddress(dto.getSenderAddress());
        bodyRequest.setSenderName(dto.getSenderName());
        bodyRequest.setBankID(dto.getBankID());
        bodyRequest.setBenAccount(dto.getBenAccount());
        bodyRequest.setIbftChannel(dto.getIbftChannel());

        NapasApiClient napasApiClient = new NapasApiClient();
        return napasApiClient.enqCheckAcc(napasHeaderRequest, bodyRequest);
    }

    // check credit account valid or not
    public boolean checkCreditAccValid(HttpServletRequest request, FTFastOutsideDTO dto) {
        try {
            if ("CARD".equals(dto.getTransferBy())) {
                String ibftChannel = StringUtils.isEmpty(dto.getIbftChannel()) ? "" : dto.getIbftChannel();
                // check type card
                EnqCheckCardResponse enqCheckCardResponse = (EnqCheckCardResponse) CacheConfiguration.getHazelcastInstance()
                        .getMap("FAST_NAME_CARD_RECEIVES").get(dto.getCreditAccount() + ibftChannel);
                if (enqCheckCardResponse == null) {
                    EnqCheckCardDTO checkCardDTO = new EnqCheckCardDTO();
                    checkCardDTO.setBenCard(dto.getCreditAccount());
                    checkCardDTO.setSenderAccount(dto.getDebitAccount());
                    checkCardDTO.setSenderAddress(swibUtils.getCustomerId(request));
                    checkCardDTO.setSenderName(dto.getSeabBenCus());
                    enqCheckCardResponse = enqCheckCard(request, checkCardDTO);
                }
                if (RESULT_OK.equals(enqCheckCardResponse.getResponseCode())
                        && dto.getSeabBenCus().trim().equals(enqCheckCardResponse.getAccountInfo().getAccountName().trim())) {
                    return true;
                }
            } else {
                // check type account
                EnqCheckAccResponse enqCheckAccResponse = null;
                List<BenBank> benBanksList = CacheConfiguration.getHazelcastInstance().getList("T24_BANK_ONLINE");
                if (benBanksList.isEmpty()) {
                    T24EnquiryApiClient apiClient = new T24EnquiryApiClient(createT24Header(request));
                    benBanksList = apiClient.getBankOnlineChannelList();
                }
                String benBankCode = "";
                for (BenBank benBank : benBanksList) {
                    if (dto.getCreditBankID().equals(benBank.getBenBankCode())) {
                        benBankCode = benBank.getBenBankID();
                        break;
                    }
                }
                String ibftChannel = StringUtils.isEmpty(dto.getIbftChannel()) ? "" : dto.getIbftChannel();
                enqCheckAccResponse = (EnqCheckAccResponse) CacheConfiguration.getHazelcastInstance()
                        .getMap("FAST_NAME_ACCOUNT_RECEIVES").get(dto.getCreditAccount() + benBankCode + ibftChannel);
                if (enqCheckAccResponse == null) {
                    EnqCheckAccountDTO checkAccountDTO = new EnqCheckAccountDTO();
                    checkAccountDTO.setBenAccount(dto.getCreditAccount());
                    checkAccountDTO.setBankID(benBankCode);
                    checkAccountDTO.setSenderAccount(dto.getDebitAccount());
                    checkAccountDTO.setSenderName(dto.getSeabBenCus());
                    checkAccountDTO.setSenderAddress(swibUtils.getCustomerId(request));
                    enqCheckAccResponse = enqCheckAcc(request, checkAccountDTO);
                }
                if (RESULT_OK.equals(enqCheckAccResponse.getResponseCode())
                        && dto.getSeabBenCus().trim().equals(enqCheckAccResponse.getAccountInfo().getAccountName().trim())) {
                    return true;
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return false;
        }
        return false;
    }

    public String getBenBankId(HttpServletRequest request, FTFastOutsideDTO dto) {
        List<BenBank> benBanksList = CacheConfiguration.getHazelcastInstance().getList("T24_BANK_ONLINE");
        if (benBanksList.isEmpty()) {
            T24EnquiryApiClient apiClient = new T24EnquiryApiClient(createT24Header(request));
            benBanksList = apiClient.getBankOnlineChannelList();
        }
        String benBankCode = "";
        for (BenBank benBank : benBanksList) {
            if (dto.getCreditBankID().equals(benBank.getBenBankCode())) {
                benBankCode = benBank.getBenBankID();
                break;
            }
        }
        return benBankCode;
    }

//    public GetTransferResDTO getTypeAmtV2(HttpServletRequest request, GetTransferReqDTO reqDTO) {
//        T24Header header = t24HeaderGetFee(request);
//
//        UserSessionInfo info = swibUtils.getUserSessionInfo(request);
//        GetTypeATMV2EnquiryDTO enquiryDTO = new GetTypeATMV2EnquiryDTO();
//        enquiryDTO.setEnqID("T24ENQ.SEAB.E.API.GET.TYPE.AMT.V2");
//        enquiryDTO.setCustomerId(swibUtils.getCustomerId(request));
//        enquiryDTO.setPartnerId(reqDTO.getPartnerId());
//        enquiryDTO.setChannel(info.getChannel());
//        enquiryDTO.setDebitAccount(reqDTO.getDebitAccount());
//        enquiryDTO.setTransactionType("1176");
//        enquiryDTO.setCreditBank(reqDTO.getCreditBankCode());
//        enquiryDTO.setTxnAmount(reqDTO.getTxnAmount());
//        enquiryDTO.setTxnCcy(reqDTO.getTxnCcy());
//        enquiryDTO.setCreditAccount(reqDTO.getCreditAccount());
//
//        T24BodyReqDTO<GetTypeATMV2EnquiryDTO> body = new T24BodyReqDTO<>();
//        body.setCommand(GET_ENQUIRY);
//        body.setEnquiry(enquiryDTO);
//
//        HashMap<String, Object> param = new HashMap<>();
//        param.put("header", header);
//        param.put("body", body);
//
//        RestTemplate restTemplate = new RestTemplate();
//        GetTransferResDTO result = new GetTransferResDTO();
//        try {
//            HttpHeaders httpHeaders = new HttpHeaders();
//            httpHeaders.setContentType(MediaType.TEXT_PLAIN);
//
//            HttpEntity<String> entity = new HttpEntity<>(JFactory.toString(param), httpHeaders);
//            ResponseEntity<T24GetTypeATMV2ResDTO> res = restTemplate.postForEntity(
//                    header.getUri(), entity, T24GetTypeATMV2ResDTO.class);
//            if (HttpStatus.OK.equals(res.getStatusCode()) && res.getBody() != null) {
//                List<T24GetTypeATMV2ResDTO.Data> lst = res.getBody().getBody().getEnquiry().getAmtV2FeeInfo();
//                T24GetTypeATMV2ResDTO.Data data = lst.get(0);
//                if (StringUtils.isEmpty(data.getErrorId())) {
//                    BeanUtils.copyProperties(data, result);
//                    result.setResponseCode(RESULT_OK);
//                }
//            }
//            if (StringUtils.isEmpty(result.getChargeAmt())) {
//                result.setErrorMessage("Get free has error");
//            }
//        } catch (Exception e) {
//            log.error(e.getMessage(), e);
//            result.setErrorMessage("Get free has error: " + e.getMessage());
//        }
//        return result;
//    }

    @SuppressWarnings("unchecked")
    public List<STOResponseDTO> getTransactionsSTO(HttpServletRequest request) throws CoreApiException {
        log.debug("request to get all Transactions");

        STO4BodyRequest bodyRequest = new STO4BodyRequest();
        bodyRequest.setAuthenType(STO4);
        bodyRequest.setCommand(GET_ENQUIRY);
        bodyRequest.setSessionId(swibUtils.getSessionId(request));
        bodyRequest.setCustomerID(swibUtils.getCustomerId(request));
        bodyRequest.setStoID("");
        bodyRequest.setCreditAccount("");
        bodyRequest.setDebitAccount("");

        List<STOResponseDTO> stoLst = new ArrayList<>();
        try {
            List<HashMap<String, String>> hashMaps = stoService.STO4(createStoHeader(request), bodyRequest);
            String str = JFactory.toString(hashMaps);
            ObjectMapper mapper = new ObjectMapper();
            mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
            stoLst.addAll(Arrays.asList(mapper.readValue(str, STOResponseDTO[].class)));
        } catch (Exception e) {
            throw new CoreApiException("P0405-SERVICE", e.getMessage(), "");
        }
        return stoLst;
    }

    /**
     * Get Beneficiary list has ben_code in lstBenTypeCode
     *
     * @param request        {@link HttpServletRequest}
     * @param lstBenTypeCode {@link List<String>}
     * @return List contact info
     * @throws Exception Core API exception
     */
    public List<ContactsInfo> getBeneficiaryList(HttpServletRequest request, List<String> lstBenTypeCode) throws Exception {
        Map<String, String> map = swibUtils.buildRequestParams(request);
        ContactsResponse res = swibCommonService.getContactsByType(map, "transfer");
        if (!RESULT_OK.equals(res.getResponseCode()) && res.getContactsInfos() == null) {
            throw new CoreApiException(res.getResponseCode(), res.getResponseMessage(), "");
        }
        if (lstBenTypeCode == null || lstBenTypeCode.isEmpty()) {
            return res.getContactsInfos();
        }

        return res.getContactsInfos().stream()
                .filter(p -> lstBenTypeCode.contains(p.getBentype_code()))
                .collect(Collectors.toList());
    }

    public FTResponseDTO stoInside(HttpServletRequest request, STOInsideReqDTO dto) throws Exception {
        StoHeaderRequest header = createStoHeader(request);
        TransInsideBodyRequest body = new TransInsideBodyRequest();
        BeanUtils.copyProperties(dto, body);
        body.setCommand(GET_TRANSACTION);
        body.setAuthenType(GET_TRANS_MAINTAIN);
        body.setSessionId(swibUtils.getSessionId(request));
        body.setCustomerID(swibUtils.getCustomerId(request));

        TransInsideResponse insideResponse = stoService.getTransInside(header, body);
        FTResponseDTO responseDTO = new FTResponseDTO();
        BeanUtils.copyProperties(insideResponse, responseDTO);

        return responseDTO;
    }

    public com.seabank.biz.sto.bean.TransOutsideResponse stoOutside(HttpServletRequest request, STOOutsideReqDTO dto) throws Exception {
        StoHeaderRequest header = createStoHeader(request);
        TransOutsideBodyRequest body = new TransOutsideBodyRequest();
        BeanUtils.copyProperties(dto, body);
        body.setCommand(GET_TRANSACTION);
        body.setAuthenType(GET_TRANS_MAINTAIN);
        body.setSessionId(swibUtils.getSessionId(request));
        body.setCustomerID(swibUtils.getCustomerId(request));

        com.seabank.biz.sto.bean.TransOutsideResponse result;
        try {
            result = stoService.getTransOutside(header, body);
        } catch (Exception e) {
            if (e instanceof CoreApiException) {
                throw e;
            } else {
                log.error(e.getMessage(), e);
                result = new com.seabank.biz.sto.bean.TransOutsideResponse();
                result.setErrorMessage("STO trans outside has error: " + e.getMessage());
            }
        }
        return result;
    }

    public List<BankInfoExt> getListBank(HttpServletRequest request) throws Exception {
        List<BankInfoExt> result = new ArrayList<>();
        List<BenBank> lstBenBank = getListBenBanks(request);
        List<BankInfo> bankInfos = swibUtilityService.getBank(false);

        List<String> lstBenBankCode = lstBenBank.stream().map(BenBank::getBenBankCode).collect(Collectors.toList());
        bankInfos.forEach(b -> {
            BankInfoExt ext = new BankInfoExt();
            try {
                BeanUtils.copyProperties(b, ext);
                if (lstBenBankCode.contains(b.getBankCode())) {
                    BenBank benBank = lstBenBank.stream()
                            .filter(bb -> bb.getBenBankCode().equals(b.getBankCode())).findFirst().orElse(null);
                    ext.setFastOutside(true);
                    assert benBank != null;
                    ext.setBenBankID(benBank.getBenBankID());
                    ext.setBenAccType(benBank.getBenAccType());
                    ext.setBenBankCode(benBank.getBenBankCode());
                    ext.setBenBankName(benBank.getBenBankName());
                    ext.setBenBankRoutingID(benBank.getBenBankRoutingID());
                    ext.setBenBankShortName(benBank.getBenBankShortName());
                    ext.setBenBankCity(benBank.getBenBankCity());
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
            result.add(ext);
        });


        return result;
    }

    private List<BenBank> getListBenBanks(HttpServletRequest request) {
        List<BenBank> benBanksList = new ArrayList<>();
        try {
            benBanksList = CacheConfiguration.getHazelcastInstance().getList("T24_BANK_ONLINE");
            if (benBanksList.isEmpty()) {
                T24Header header = createT24Header(request);
                T24EnquiryApiClient apiClient = new T24EnquiryApiClient(header);
                benBanksList = apiClient.getBankOnlineChannelList();
            }
        } catch (Exception e) {
            log.error("Get list clr banks has error: " + e.getMessage(), e);
        }
        return benBanksList;
    }

//    private boolean checkFTOutsideAcc(HttpServletRequest request, FTOutsideDTO dto) {
//        EnqCheckAccountDTO checkDTO = new EnqCheckAccountDTO();
//        checkDTO.setSenderAccount(dto.getDebitAccount());
//        checkDTO.setSenderName(swibUtils.getUserName(request));
//        checkDTO.setSenderAddress(swibUtils.getCustomerId(request));
//        checkDTO.setBenAccount(dto.getCreditAccount());
//        checkDTO.setBankID(dto.getCreditBankId());
//
//        EnqCheckAccResponse rCheck = enqCheckAcc(request, checkDTO);
//        return rCheck != null;
//    }

    private NapasHeaderRequest createNapasHeader(HttpServletRequest request) {
        NapasHeaderRequest napasHeaderRequest = new NapasHeaderRequest();
        UserSessionInfo info = swibUtils.getUserSessionInfo(request);
        //DPW
        napasHeaderRequest.setxIBMClientId(properties.getDpw().getHeader().getClientId());
        napasHeaderRequest.setxIBMClientSecret(properties.getDpw().getHeader().getSecret());

        //common header
        napasHeaderRequest.setReqType(properties.getCommon().getHeader().getReqType());
        napasHeaderRequest.setChannel(info.getChannel());
        napasHeaderRequest.setTrusted(properties.getCommon().getHeader().getTrusted());

        //Napas header
        napasHeaderRequest.setUri(properties.getNapas().getHeader().getUri());
        napasHeaderRequest.setApi(properties.getNapas().getHeader().getApi());
        napasHeaderRequest.setApiKey(properties.getNapas().getHeader().getApiKey());
        napasHeaderRequest.setPriority(properties.getNapas().getHeader().getPriority());
        napasHeaderRequest.setSubChannel(info.getSubChannel());
        napasHeaderRequest.setRequestAPI(properties.getNapas().getHeader().getRequestAPI());
        napasHeaderRequest.setRequestNode(properties.getNapas().getHeader().getRequestNode());

        napasHeaderRequest.setLocation(info.getLocation());
        napasHeaderRequest.setUserID(info.getUsername());
        napasHeaderRequest.setContext(info.getContext());
        return napasHeaderRequest;
    }

    private HashMap<String, String> creatParamCustomerAcc(HttpServletRequest request) {
        return new HashMap<String, String>() {{
            put(SwibConstants.API_HEADER_USER_ID, swibUtils.getUserName(request));
            put(SwibConstants.API_HEADER_CUSTOMER_ID, swibUtils.getCustomerId(request));
        }};
    }

    private FastOutsideHeader createFastOutSideHeader(HttpServletRequest request) {
        UserSessionInfo info = swibUtils.getUserSessionInfo(request);

        FastOutsideHeader fastOutsideHeader = new FastOutsideHeader();

        //DPW
        fastOutsideHeader.setxIBMClientId(properties.getDpw().getHeader().getClientId());
        fastOutsideHeader.setxIBMClientSecret(properties.getDpw().getHeader().getSecret());
        //common header
        fastOutsideHeader.setReqType(properties.getCommon().getHeader().getReqType());
        fastOutsideHeader.setTrusted(properties.getCommon().getHeader().getTrusted());
        fastOutsideHeader.setChannel(info.getChannel());
        fastOutsideHeader.setSubChannel(info.getSubChannel());

        //account API header
        fastOutsideHeader.setApi(properties.getFastTransfer().getHeader().getApi());
        fastOutsideHeader.setApiKey(properties.getFastTransfer().getHeader().getApiKey());
        fastOutsideHeader.setUri(properties.getFastTransfer().getHeader().getUri());
        fastOutsideHeader.setPriority(properties.getFastTransfer().getHeader().getPriority());
        fastOutsideHeader.setRequestAPI(properties.getFastTransfer().getHeader().getRequestAPI());
        fastOutsideHeader.setRequestNode(properties.getFastTransfer().getHeader().getRequestNode());

        fastOutsideHeader.setLocation(info.getLocation());
        fastOutsideHeader.setUserID(info.getUsername());
        fastOutsideHeader.setContext(info.getContext());

        return fastOutsideHeader;
    }

    private FTInsideHeader createFTInsideHeader(HttpServletRequest request) {
        UserSessionInfo info = swibUtils.getUserSessionInfo(request);


        FTInsideHeader ftInsideHeader = new FTInsideHeader();
        //DPW
        ftInsideHeader.setxIBMClientId(properties.getDpw().getHeader().getClientId());
        ftInsideHeader.setxIBMClientSecret(properties.getDpw().getHeader().getSecret());

        //common header
        ftInsideHeader.setReqType(properties.getCommon().getHeader().getReqType());
        ftInsideHeader.setTrusted(properties.getCommon().getHeader().getTrusted());
        ftInsideHeader.setChannel(info.getChannel());
        ftInsideHeader.setSubChannel(info.getSubChannel());

        //account API header
        ftInsideHeader.setApi(properties.getFtinside().getHeader().getApi());
        ftInsideHeader.setApiKey(properties.getFtinside().getHeader().getApiKey());
        ftInsideHeader.setUri(properties.getFtinside().getHeader().getUri());
        ftInsideHeader.setPriority(properties.getFtinside().getHeader().getPriority());
        ftInsideHeader.setRequestAPI(properties.getFtinside().getHeader().getRequestAPI());
        ftInsideHeader.setRequestNode(properties.getFtinside().getHeader().getRequestNode());

        ftInsideHeader.setContext(info.getContext());
        ftInsideHeader.setUserID(info.getUsername());
        ftInsideHeader.setLocation(info.getLocation());

        return ftInsideHeader;
    }

    private FTOutsideHeader createFTOHeader(HttpServletRequest request) {
        UserSessionInfo info = swibUtils.getUserSessionInfo(request);
        FTOutsideHeader ftOutsideHeader = new FTOutsideHeader();
        //DPW
        ftOutsideHeader.setxIBMClientId(properties.getDpw().getHeader().getClientId());
        ftOutsideHeader.setxIBMClientSecret(properties.getDpw().getHeader().getSecret());

        //common header
        ftOutsideHeader.setReqType(properties.getCommon().getHeader().getReqType());
        ftOutsideHeader.setTrusted(properties.getCommon().getHeader().getTrusted());
        ftOutsideHeader.setChannel(info.getChannel());
        //account API header
        ftOutsideHeader.setSubChannel(info.getSubChannel());
        ftOutsideHeader.setApi(properties.getFtOutside().getHeader().getApi());
        ftOutsideHeader.setApiKey(properties.getFtOutside().getHeader().getApiKey());
        ftOutsideHeader.setUri(properties.getFtOutside().getHeader().getUri());
        ftOutsideHeader.setPriority(properties.getFtOutside().getHeader().getPriority());
        ftOutsideHeader.setRequestAPI(properties.getFtOutside().getHeader().getRequestAPI());
        ftOutsideHeader.setRequestNode(properties.getFtOutside().getHeader().getRequestNode());

        ftOutsideHeader.setUserID(info.getUsername());
        ftOutsideHeader.setContext(info.getContext());
        ftOutsideHeader.setLocation(info.getLocation());
        return ftOutsideHeader;
    }

    private T24Header createT24Header(HttpServletRequest request) {
        UserSessionInfo info = swibUtils.getUserSessionInfo(request);
        T24Header header = new T24Header();

        //DPW
        header.setxIBMClientId(properties.getDpw().getHeader().getClientId());
        header.setxIBMClientSecret(properties.getDpw().getHeader().getSecret());


        //common header
        header.setReqType(properties.getCommon().getHeader().getReqType());
        header.setTrusted(properties.getCommon().getHeader().getTrusted());
        header.setChannel(info.getChannel());
        header.setSubChannel(info.getSubChannel());
        header.setRequestAPI(properties.getCommon().getHeader().getRequestAPI());
        header.setRequestNode(properties.getCommon().getHeader().getRequestNode());

        //T24Header API header
        header.setApi(properties.getAccount().getHeader().getApi());
        header.setApiKey(properties.getAccount().getHeader().getApiKey());
        header.setUri(properties.getAccount().getHeader().getUri());
        header.setPriority(properties.getAccount().getHeader().getPriority());

        header.setLocation(info.getLocation());
        header.setUserID(info.getUsername());
        header.setContext(info.getContext());

        return header;
    }

    private T24Header t24HeaderGetFee(HttpServletRequest request) {
        UserSessionInfo info = swibUtils.getUserSessionInfo(request);
        T24Header header = new T24Header();

        //DPW
        header.setxIBMClientId(properties.getDpw().getHeader().getClientId());
        header.setxIBMClientSecret(properties.getDpw().getHeader().getSecret());


        //common header
        header.setReqType(properties.getCommon().getHeader().getReqType());
        header.setTrusted(properties.getCommon().getHeader().getTrusted());
        header.setChannel(info.getChannel());
        header.setSubChannel(info.getSubChannel());
        header.setRequestAPI(properties.getCommon().getHeader().getRequestAPI());
        header.setRequestNode(properties.getCommon().getHeader().getRequestNode());

        //T24Header API header
        header.setApi(properties.getGetTypeAmtV2().getHeader().getApi());
        header.setApiKey(properties.getGetTypeAmtV2().getHeader().getApiKey());
        header.setUri(properties.getGetTypeAmtV2().getHeader().getUri());
        header.setPriority(properties.getGetTypeAmtV2().getHeader().getPriority());

        header.setLocation(info.getLocation());
        header.setUserID(info.getUsername());
        header.setContext(info.getContext());

        return header;
    }

    public StoHeaderRequest createStoHeader(HttpServletRequest request) {
        StoHeaderRequest headerRequest = new StoHeaderRequest();
        UserSessionInfo info = swibUtils.getUserSessionInfo(request);

        ApiClientProperties.APIClientHeader header = properties.getSto().getHeader();
        headerRequest.setReqType(header.getReqType());
        headerRequest.setApi(header.getApi());
        headerRequest.setApiKey(header.getApiKey());
        headerRequest.setPriority(header.getPriority());
        headerRequest.setChannel(info.getChannel());
        headerRequest.setSubChannel(info.getSubChannel());
        headerRequest.setLocation(info.getLocation());
        headerRequest.setContext(info.getContext());
        headerRequest.setTrusted(header.getTrusted());
        headerRequest.setUserID(info.getUsername());
        headerRequest.setRequestAPI(header.getRequestAPI());
        headerRequest.setRequestNode(header.getRequestNode());
        headerRequest.setUri(header.getUri());

        ApiClientProperties.DpwHeader dpwHeader = properties.getDpw().getHeader();
        headerRequest.setxIBMClientId(dpwHeader.getClientId());
        headerRequest.setxIBMClientSecret(dpwHeader.getSecret());

        return headerRequest;
    }

    private void saveTransferInside(FTInsideTransactionParam ftInsideTransactionParam) {
        try {
            FtAccount account = new FtAccount();
            account.setId(ftInsideTransactionParam.getCreditAccount() +
                    ftInsideTransactionParam.getAccountName() + ftInsideTransactionParam.getCustomerID());
            account.setAccId(ftInsideTransactionParam.getCreditAccount());
            account.setAccName(ftInsideTransactionParam.getAccountName());
            account.setFtType(FT_ACCOUNT_NUMBER);
            account.setCusId(ftInsideTransactionParam.getCustomerID());
            account.setCreatedDate(new Timestamp(System.currentTimeMillis()));
            swibCommonService.saveAccountTransfer(account);
        } catch (Exception e) {
            LogUtil.exception(log, e, ftInsideTransactionParam.getCustomerID());
        }
    }

    private void saveTransferFastOutside(BodyGetITransUnTrusted param, String transferType) {
        try {
            FtAccount account = new FtAccount();
            account.setId(param.getCreditAccount() + param.getCreditAccountName() + param.getCustomerID() + transferType);
            account.setAccId(param.getCreditAccount());
            account.setAccName(param.getCreditAccountName());
            account.setFtType(transferType);
            account.setCusId(param.getCustomerID());
            account.setBankId(param.getCreditBankID());
            account.setCreatedDate(new Timestamp(System.currentTimeMillis()));
            swibCommonService.saveAccountTransfer(account);
        } catch (Exception e) {
            log.error("saveTransferFastOutside has error: " + e.getMessage(), e);
        }
    }

    private void saveTransferOutSide(TransUnTrustedBodyRequest param) {
        try {
            FtAccount account = new FtAccount();
            account.setId(param.getCreditAccount() + param.getCreditAccountName() + param.getCustomerID());
            account.setAccId(param.getCreditAccount());
            account.setAccName(param.getCreditAccountName());
            account.setFtType(FT_ACCOUNT_NUMBER);
            account.setCusId(param.getCustomerID());
            account.setBankId(param.getCreditBankId());
            account.setCityId(param.getCreditProvince());
            account.setBranchId(param.getCreditBranchId());
            account.setRoomNo(param.getPgd());
            account.setBranchName(param.getCreditBranchName());
            account.setCreatedDate(new Timestamp(System.currentTimeMillis()));
            swibCommonService.saveAccountTransfer(account);
        } catch (Exception e) {
            log.error("saveTransferOutSide has error: " + e.getMessage(), e);
        }
    }
}
