package vn.com.seabank.biz.web.rest;

import com.seabank.biz.custinfo.bean.AccountInfo;
import com.swib.seanet.napas.bean.EnqCheckAccResponse;
import com.swib.seanet.napas.bean.EnqCheckCardResponse;
import com.swib.services.config.CacheConfiguration;
import com.swib.services.service.SWIBEnquiryService;
import com.swib.services.utils.SwibUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import vn.com.seabank.biz.domain.UserSessionInfo;
import vn.com.seabank.biz.dto.EnqCheckAccountDTO;
import vn.com.seabank.biz.dto.EnqCheckCardDTO;
import vn.com.seabank.biz.dto.ResponseObject;
import vn.com.seabank.biz.exception.CoreApiException;
import vn.com.seabank.biz.service.P0405Service;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import static vn.com.seabank.biz.ultil.ErrorMessage.INTERNAL_SERVER_ERROR;

@RestController
@RequestMapping("/api/common")
public class CommonResource {

    private final Logger log = LoggerFactory.getLogger(CommonResource.class);

    private static final String RESULT_OK = "00";
    private static final int TIME_FOR_CACHE = 10;

    private final SwibUtils swibUtils;
    private final P0405Service service;
    private final SWIBEnquiryService swibEnquiryService;

    public CommonResource(SwibUtils swibUtils,
                          P0405Service service,
                          SWIBEnquiryService swibEnquiryService) {
        this.swibUtils = swibUtils;
        this.service = service;
        this.swibEnquiryService = swibEnquiryService;
    }

    /**
     * Get list account source
     *
     * @param request      {@link HttpServletRequest}
     * @param typeTransfer - Type transfer
     * @return List resource account
     */
    @GetMapping(value = "/account-info/{typeTransfer}")
    @SuppressWarnings("unchecked")
    public ResponseEntity<ResponseObject<?>> getAccountInfo(HttpServletRequest request,
                                                            @PathVariable String typeTransfer) {
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        Map<String, Object> listAcc = swibEnquiryService.getGroupAccountsForTransfer(userSessionInfo, typeTransfer);
        if (listAcc == null) {
            throw new CoreApiException(INTERNAL_SERVER_ERROR, "P0405_FT");
        }
        return new ResponseEntity<>(ResponseObject.success(listAcc), HttpStatus.OK);
    }

    @PostMapping(value = "/enq-check-acc")
    public ResponseEntity<ResponseObject<EnqCheckAccResponse>> enqCheckAcc(HttpServletRequest request,
                                                                           @RequestBody EnqCheckAccountDTO dto) throws IOException, CoreApiException {
        String ibftChannel = StringUtils.isEmpty(dto.getIbftChannel()) ? "" : dto.getIbftChannel();
        EnqCheckAccResponse enqCheckAccResponse;
        enqCheckAccResponse = (EnqCheckAccResponse) CacheConfiguration.getHazelcastInstance().getMap("FAST_NAME_ACCOUNT_RECEIVES")
                .get(dto.getBenAccount() + dto.getBankID() + ibftChannel);
        if (enqCheckAccResponse == null) {
            enqCheckAccResponse = service.enqCheckAcc(request, dto);
            if (RESULT_OK.equals(enqCheckAccResponse.getResponseCode())) {
                CacheConfiguration.getHazelcastInstance().getMap("FAST_NAME_ACCOUNT_RECEIVES")
                        .put(dto.getBenAccount() + dto.getBankID() + ibftChannel,
                                enqCheckAccResponse, TIME_FOR_CACHE, TimeUnit.MINUTES);
            }
        }
        return new ResponseEntity<>(ResponseObject.success(enqCheckAccResponse), HttpStatus.OK);
    }

    /**
     * Get all get name card.
     */
    @PostMapping(value = "/enq-check-card")
    public ResponseEntity<ResponseObject<EnqCheckCardResponse>> enqCheckCard(HttpServletRequest request,
                                                                             @RequestBody EnqCheckCardDTO dto) throws IOException, CoreApiException {
        log.debug("REST request to get a all get name card");
        String ibftChannel = StringUtils.isEmpty(dto.getIbftChannel()) ? "" : dto.getIbftChannel();
        EnqCheckCardResponse enqCheckCardResponse;
        enqCheckCardResponse = (EnqCheckCardResponse) CacheConfiguration.getHazelcastInstance()
                .getMap("FAST_NAME_CARD_RECEIVES").get(dto.getBenCard() + ibftChannel);
        if (enqCheckCardResponse == null) {
            enqCheckCardResponse = service.enqCheckCard(request, dto);
            if (RESULT_OK.equals(enqCheckCardResponse.getResponseCode())) {
                CacheConfiguration.getHazelcastInstance().getMap("FAST_NAME_CARD_RECEIVES")
                        .put(dto.getBenCard() + ibftChannel, enqCheckCardResponse, TIME_FOR_CACHE, TimeUnit.MINUTES);
            }
        }
        return new ResponseEntity<>(ResponseObject.success(enqCheckCardResponse), HttpStatus.OK);
    }

}
