package vn.com.seabank.biz.web.rest;

import com.google.zxing.BinaryBitmap;
import com.google.zxing.MultiFormatReader;
import com.google.zxing.Result;
import com.google.zxing.client.j2se.BufferedImageLuminanceSource;
import com.google.zxing.common.HybridBinarizer;
import com.hazelcast.core.HazelcastInstance;
import com.seabank.biz.custinfo.bean.CustInfoAc3Response;
import com.seabank.biz.pfm.dto.request.InsertTransactionOnlineReq;
import com.swib.seanet.ben.bean.ContactsInfo;
import com.swib.services.config.ApiClientProperties;
import com.swib.services.config.CacheConfiguration;
import com.swib.services.domain.VoucherBatch;
import com.swib.services.service.*;
import com.swib.services.service.dto.UseVoucherRequestDTO;
import com.swib.services.utils.SwibUtils;
import com.swib.services.validators.CurrentAccountValidator;
import com.swib.services.validators.PersonalValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import vn.com.seabank.biz.domain.FtCfgCache;
import vn.com.seabank.biz.domain.UserSessionInfo;
import vn.com.seabank.biz.dto.*;
import vn.com.seabank.biz.exception.CoreApiException;
import vn.com.seabank.biz.exception.NextgenApiException;
import vn.com.seabank.biz.service.P0405Service;

import javax.imageio.ImageIO;
import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import static vn.com.seabank.biz.exception.CoreApiException.PrefixErrorCode.FASTOUTSIDE;
import static vn.com.seabank.biz.ultil.ErrorMessage.FASTOUTSIDE_ACC_RECEIVED_INVALID;
import static vn.com.seabank.biz.ultil.ErrorMessage.PARAMETER_HAS_ERROR;

@RestController
@RequestMapping("/api")
public class P0405Resource {
    private static final String VND = "VND";

    private final Logger log = LoggerFactory.getLogger(P0405Resource.class);

    private final SwibUtils swibUtils;
    private final P0405Service service;
    private final CurrentAccountValidator validator;
    private final String PREFIX = "P0405-FT";
    private final ApiClientProperties properties;
    private final SWIBEnquiryService swibEnquiryService;
    private final HazelcastInstance hazelcastInstance = CacheConfiguration.getHazelcastInstance();

    private final String FT_BY_ACCOUNT = "FT_BY_ACCOUNT";
    private final String FT_BY_CARD = "FT_BY_CARD";
    private static final String CODE02 = "02";
    private static final String BANKAPI_ADMIN_VOUCHER = "BANKAPI-ADMIN-VOUCHER";
    private static final String CODE99 = "099";
    private static final String BANKAPI_NEXTGEN = "BANKAPI-NEXTGEN";
    // Time cache
    private static final int TIME_FOR_CACHE = 30;


    @Autowired
    VoucherBatchService voucherBatchService;

    @Autowired
    SwibCommonService swibCommonService;

    @Autowired
    PrePaidAccService prePaidAccService;

    @Inject
    private CustomerObserver customerObserver;

    public P0405Resource(SwibUtils swibUtils,
                         P0405Service service,
                         CurrentAccountValidator validator,
                         ApiClientProperties properties,
                         SWIBEnquiryService swibEnquiryService) {
        this.swibUtils = swibUtils;
        this.service = service;
        this.validator = validator;
        this.properties = properties;
        this.swibEnquiryService = swibEnquiryService;
    }

    @PostMapping(value = "/ft-inside")
    public ResponseEntity<ResponseObject<FTResponseDTO>> ftInside(HttpServletRequest request,
                                                                  @RequestBody FTInsideDTO dto) throws CoreApiException {
        log.info("-----> START FT INSIDE:: " + dto.toString());
        VoucherBatch validVoucher = null;
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);

        if (prePaidAccService.checkLimitRemainPrePaidCard(dto.getDebitAccount(), Long.valueOf(dto.getTransAmount()))) {
            if (!StringUtils.isEmpty(dto.getRegIssuanceName())) {
                validVoucher = voucherBatchService.getValidVoucher(request, dto.getRegIssuanceName(), FT_BY_ACCOUNT);
                if (validVoucher == null) {
                    throw new CoreApiException(CODE02, "EVoucherCode is not exist or wrong type", BANKAPI_ADMIN_VOUCHER);
                }
            }

            PersonalValidator.Result rs = validator.validate((CurrentAccountValidator.ICurrentAccountValidator) () -> {
                List<String> accounts = new ArrayList<>();
                accounts.add(dto.getDebitAccount());
                return accounts;
            }, request);
            if (!rs.isSuccess()) {
                return new ResponseEntity<>(
                        ResponseObject.error("P0405-FT-INSIDE", "PersonalValidator has error: " + rs.getErrorDesc()), HttpStatus.OK);
            }

            FTResponseDTO response = service.ftInside(request, dto);
            customerObserver.onAccountChanged(userSessionInfo);
            try {
                if (validVoucher != null) {
                    UseVoucherRequestDTO useVoucherRequestDTO = new UseVoucherRequestDTO();
                    useVoucherRequestDTO.setAccountNo(dto.getAccountNo());
                    useVoucherRequestDTO.setCreditCcy(dto.getCreditCcy());
                    useVoucherRequestDTO.setDebitCcy(dto.getDebitCcy());
                    useVoucherRequestDTO.setEvoucherCode(validVoucher.getEvoucherCode());
                    useVoucherRequestDTO.setTransId(response.getTransactionNO());
                    useVoucherRequestDTO.setTransAmt(dto.getTransAmount());
                    useVoucherRequestDTO.setTransTypeReq(FT_BY_ACCOUNT);
                    voucherBatchService.usedVoucher(request, useVoucherRequestDTO);
                }
                if (!StringUtils.isEmpty(dto.getCategoryId())) {
                    InsertTransactionOnlineReq insertTransactionOnlineReq = new InsertTransactionOnlineReq();
                    insertTransactionOnlineReq.setCategoryId(dto.getCategoryId());
                    insertTransactionOnlineReq.setFtId(response.getTransactionNO());
                    insertTransactionOnlineReq.setCustomerId(swibUtils.getCustomerId(request));
                    insertTransactionOnlineReq.setTransDate(dateToString(new Date()));
                    swibCommonService.insertTransactionOnline(userSessionInfo, insertTransactionOnlineReq);
                }
            } catch (Exception ex) {
                log.error("-----> FT INSIDE ERROR:: ", ex.getMessage());
            }
            log.info("-----> FT INSIDE SUCCESS:: " + response.getTransactionNO());
            return new ResponseEntity<>(ResponseObject.success(response), HttpStatus.OK);
        } else {
            throw new CoreApiException(CODE99, "Prepaid card remain amount not enough", BANKAPI_NEXTGEN);
        }
    }

    @PostMapping(value = "/ft-outside")
    public ResponseEntity<ResponseObject<FTResponseDTO>> ftOutside(HttpServletRequest request,
                                                                   @RequestBody FTOutsideDTO dto) throws CoreApiException {
        log.info("-----> START FT OUTSIDE:: " + dto.toString());
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);

        if (prePaidAccService.checkLimitRemainPrePaidCard(dto.getDebitAccount(), Long.valueOf(dto.getTransAmount()))) {
            VoucherBatch validVoucher = null;
            if (!StringUtils.isEmpty(dto.getRegIssuanceName())) {
                validVoucher = voucherBatchService.getValidVoucher(request, dto.getRegIssuanceName(), FT_BY_ACCOUNT);
                if (validVoucher == null) {
                    throw new CoreApiException(CODE02, "EVoucherCode is not exist or wrong type", BANKAPI_ADMIN_VOUCHER);
                }
            }

            PersonalValidator.Result rs = validator.validate((CurrentAccountValidator.ICurrentAccountValidator) () -> {
                List<String> accounts = new ArrayList<>();
                if (dto != null) {
                    accounts.add(dto.getDebitAccount());
                }
                return accounts;
            }, request);
            if (!rs.isSuccess()) {
                return new ResponseEntity<>(
                        ResponseObject.error("P0405-FT-OUTSIDE", "PersonalValidator has error: " + rs.getErrorDesc()), HttpStatus.OK);
            }
            FTResponseDTO res = service.ftOutside(request, dto);
            customerObserver.onAccountChanged(userSessionInfo);

            try {
                if (validVoucher != null) {
                    UseVoucherRequestDTO useVoucherRequestDTO = new UseVoucherRequestDTO();
                    useVoucherRequestDTO.setAccountNo(dto.getAccountNo());
                    useVoucherRequestDTO.setCreditCcy(dto.getCreditCcy());
                    useVoucherRequestDTO.setDebitCcy(dto.getDebitCcy());
                    useVoucherRequestDTO.setEvoucherCode(validVoucher.getEvoucherCode());
                    useVoucherRequestDTO.setTransId("CASHBACK" + res.getTransactionNO());
                    useVoucherRequestDTO.setTransAmt(dto.getTransAmount());
                    useVoucherRequestDTO.setTransTypeReq(FT_BY_ACCOUNT);
                    voucherBatchService.usedVoucher(request, useVoucherRequestDTO);
                }
                if (!StringUtils.isEmpty(dto.getCategoryId())) {
                    InsertTransactionOnlineReq insertTransactionOnlineReq = new InsertTransactionOnlineReq();
                    insertTransactionOnlineReq.setCategoryId(dto.getCategoryId());
                    insertTransactionOnlineReq.setFtId(res.getTransactionNO());
                    insertTransactionOnlineReq.setCustomerId(swibUtils.getCustomerId(request));
                    insertTransactionOnlineReq.setTransDate(dateToString(new Date()));
                    swibCommonService.insertTransactionOnline(userSessionInfo, insertTransactionOnlineReq);
                }
            } catch (Exception ex) {
                log.error("-----> FT OUTSIDE ERROR:: ", ex.getMessage());
            }
            log.info("-----> FT OUTSIDE SUCCESS:: " + res.getTransactionNO());
            return new ResponseEntity<>(ResponseObject.success(res), HttpStatus.OK);
        } else {
            throw new CoreApiException(CODE99, "Prepaid card remain amount not enough", BANKAPI_NEXTGEN);
        }
    }

    @PostMapping(value = "/ft-fast-outside")
    public ResponseEntity<ResponseObject<FTResponseDTO>> ftFastOutside(HttpServletRequest request,
                                                                       @RequestBody FTFastOutsideDTO dto) {
        log.info("-----> START FT FAST OUTSIDE:: " + dto.toString());
        VoucherBatch validVoucher = null;
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        if (prePaidAccService.checkLimitRemainPrePaidCard(dto.getDebitAccount(), Long.valueOf(dto.getTransAmount()))) {
            if (!StringUtils.isEmpty(dto.getRegIssuanceName())) {
                validVoucher = voucherBatchService.getValidVoucher(request, dto.getRegIssuanceName(), FT_BY_ACCOUNT);
                if (validVoucher == null) {
                    throw new CoreApiException(CODE02, "EVoucherCode is not exist or wrong type", BANKAPI_ADMIN_VOUCHER);
                }
            }

            PersonalValidator.Result rs = validator.validate((CurrentAccountValidator.ICurrentAccountValidator) () -> {
                List<String> accounts = new ArrayList<>();
                if (dto != null) {
                    accounts.add(dto.getDebitAccount());
                }
                return accounts;
            }, request);
            if (!rs.isSuccess()) {
                throw new CoreApiException(PARAMETER_HAS_ERROR, PREFIX, rs);
            }
            if (Long.parseLong(dto.getTransAmount()) >= properties.getMaxAmount()) {
                throw new CoreApiException("MAX-AMOUNT", String.format("Amount is greater than %s", properties.getMaxAmount()), FASTOUTSIDE);
            }

            // check credit account valid or not
            if (!service.checkCreditAccValid(request, dto)) {
                log.error("Check credit account or card has error");
//                return new ResponseEntity<>(
//                        ResponseObject.error("P0405-FT-FAST-OUTSIDE", "Check credit account or card has error"), HttpStatus.OK);
                throw new NextgenApiException(FASTOUTSIDE_ACC_RECEIVED_INVALID);
            }

            FTResponseDTO responseDTO = service.ftFastOutside(request, dto);
            // clear cache enq check
            String ibftChannel = org.apache.commons.lang3.StringUtils.isEmpty(dto.getIbftChannel()) ? "" : dto.getIbftChannel();
            if ("CARD".equals(dto.getTransferBy())) {
                CacheConfiguration.getHazelcastInstance()
                        .getMap("FAST_NAME_CARD_RECEIVES").remove(dto.getCreditAccount() + ibftChannel);
            } else {
                String benBankCode = service.getBenBankId(request, dto);
                CacheConfiguration.getHazelcastInstance()
                        .getMap("FAST_NAME_ACCOUNT_RECEIVES").remove(dto.getCreditAccount() + benBankCode + ibftChannel);
            }
            customerObserver.onAccountChanged(userSessionInfo);
            try {
                if (validVoucher != null) {
                    UseVoucherRequestDTO useVoucherRequestDTO = new UseVoucherRequestDTO();
                    useVoucherRequestDTO.setAccountNo(dto.getDebitAccount());
                    useVoucherRequestDTO.setCreditCcy(dto.getDebitCcy());
                    useVoucherRequestDTO.setDebitCcy(dto.getDebitCcy());
                    useVoucherRequestDTO.setEvoucherCode(validVoucher.getEvoucherCode());
                    useVoucherRequestDTO.setTransId(responseDTO.getTransactionNO());
                    useVoucherRequestDTO.setTransAmt(dto.getTransAmount());
                    useVoucherRequestDTO.setTransTypeReq(FT_BY_CARD);
                    voucherBatchService.usedVoucher(request, useVoucherRequestDTO);
                }
                if (!StringUtils.isEmpty(dto.getCategoryId())) {
                    InsertTransactionOnlineReq insertTransactionOnlineReq = new InsertTransactionOnlineReq();
                    insertTransactionOnlineReq.setCategoryId(dto.getCategoryId());
                    insertTransactionOnlineReq.setFtId(responseDTO.getTransactionNO());
                    insertTransactionOnlineReq.setCustomerId(swibUtils.getCustomerId(request));
                    insertTransactionOnlineReq.setTransDate(dateToString(new Date()));
                    swibCommonService.insertTransactionOnline(userSessionInfo, insertTransactionOnlineReq);
                }
            } catch (Exception ex) {
                log.error("-----> FT FAST OUTSIDE ERROR:: ", ex.getMessage());
            }
            log.info("-----> FT FAST OUTSIDE SUCCESS:: " + responseDTO.getTransactionNO());
            return new ResponseEntity<>(ResponseObject.success(responseDTO), HttpStatus.OK);
        } else {
            throw new CoreApiException(CODE99, "Prepaid card remain amount not enough", BANKAPI_NEXTGEN);
        }
    }

//    @PostMapping(value = "/get-transfer-fee")
//    public ResponseEntity<ResponseObject<GetTransferResDTO>> getTransferFee(HttpServletRequest request,
//                                                            @RequestBody GetTransferReqDTO dto) {
//        log.debug("get_transfer_fee: {}", JFactory.toString(dto));
//        GetTransferResDTO resDTO = service.getTypeAmtV2(request, dto);
//        return new ResponseEntity<>(ResponseObject.success(resDTO), HttpStatus.OK);
//    }

    @GetMapping(value = "/get-transactions-sto")
    public ResponseEntity<ResponseObject<List<STOResponseDTO>>> getTransactionsSto(HttpServletRequest request) throws CoreApiException {
        log.debug("get_transactions_sto: {}", swibUtils.getCustomerId(request));
        List<STOResponseDTO> result = service.getTransactionsSTO(request);
        return new ResponseEntity<>(ResponseObject.success(result), HttpStatus.OK);
    }

    @GetMapping(value = "/get-acc-beneficiary-list")
    public ResponseEntity<ResponseObject<List<ContactsInfo>>> getAccBeneficiaryList(HttpServletRequest request) throws Exception {
        log.debug("get_acc_beneficiary_list: {}", swibUtils.getCustomerId(request));
        List<ContactsInfo> result = service.getBeneficiaryList(request, Arrays.asList("BenOutT", "BenOutF", "BenInSB"));
        return new ResponseEntity<>(ResponseObject.success(result), HttpStatus.OK);
    }

    @GetMapping(value = "/get-sto-beneficiary-list")
    public ResponseEntity<ResponseObject<List<ContactsInfo>>> getStoBeneficiaryList(HttpServletRequest request) throws Exception {
        log.debug("get_sto_beneficiary_list: {}", swibUtils.getCustomerId(request));
        List<ContactsInfo> result = service.getBeneficiaryList(request, Arrays.asList("BenPhone", "BenCard", "BenOutF", "BenInSB"));
        return new ResponseEntity<>(ResponseObject.success(result), HttpStatus.OK);
    }

    @PostMapping(value = "/sto-transfer-inside")
    public ResponseEntity<ResponseObject<FTResponseDTO>> stoTransferInside(HttpServletRequest request,
                                                                           @RequestBody STOInsideReqDTO dto) throws Exception {
        FTResponseDTO res = service.stoInside(request, dto);
        return new ResponseEntity<>(ResponseObject.success(res), HttpStatus.OK);
    }

    @PostMapping(value = "/sto-transfer-outside")
    public ResponseEntity<ResponseObject<?>> stoTransferOutside(HttpServletRequest request,
                                                                @RequestBody STOOutsideReqDTO dto) throws Exception {
        com.seabank.biz.sto.bean.TransOutsideResponse resDTO = service.stoOutside(request, dto);
        return new ResponseEntity<>(ResponseObject.success(resDTO), HttpStatus.OK);
    }

    /**
     * Lấy danh sách ngân hàng tham gia chuyển tiền nhanh
     *
     * @param request {@link HttpServletRequest}
     * @return com.swib.seanet.t24enquiry.bean.BenBank
     */
    @GetMapping(value = "/get-list-bank")
    public ResponseEntity<ResponseObject<List<BankInfoExt>>> getListBank(HttpServletRequest request) throws Exception {
        log.debug("get_list_bank");
        List<BankInfoExt> res = service.getListBank(request);
        return new ResponseEntity<>(ResponseObject.success(res), HttpStatus.OK);
    }

    @PostMapping(value = "/upload-file")
    public ResponseEntity<ResponseObject<CustInfoAc3Response>> uploadFile(HttpServletRequest request, @RequestParam("file") MultipartFile file) throws Exception {
        log.debug("upload_file:" + file.getOriginalFilename());

        InputStream in = new ByteArrayInputStream(file.getBytes());
        BinaryBitmap binaryBitmap = new BinaryBitmap(
                new HybridBinarizer(new BufferedImageLuminanceSource(ImageIO.read(in))));
        Result result = new MultiFormatReader().decode(binaryBitmap);
        String accountNo = result.getText();

        CustInfoAc3Response custInfoAc3Response;
        custInfoAc3Response = (CustInfoAc3Response) CacheConfiguration.getHazelcastInstance().getMap("NAME_ACCOUNT_RECEIVES").get(accountNo);
        if (custInfoAc3Response == null) {
            custInfoAc3Response = this.swibEnquiryService.getBeneficiaryAccountInfoInside(this.swibUtils.getUserSessionInfo(request), result.getText());
            if ("00".equals(custInfoAc3Response.getResponseCode())) {
                CacheConfiguration.getHazelcastInstance().getMap("NAME_ACCOUNT_RECEIVES").put(accountNo, custInfoAc3Response, TIME_FOR_CACHE, TimeUnit.MINUTES);
            }
        }

        return new ResponseEntity<>(ResponseObject.success(custInfoAc3Response), HttpStatus.OK);
    }

    @RequestMapping(value = "/config-transfers", method = RequestMethod.GET)
    @SuppressWarnings("unchecked")
    public ResponseEntity<ResponseObject<List<FtCfgCache>>> getConfigTransfer() {
        List<FtCfgCache> ruleConfig = (List<FtCfgCache>) hazelcastInstance.getMap("ADMIN").get("TRANSFER_CACHE");
        return new ResponseEntity<>(ResponseObject.success(ruleConfig), HttpStatus.OK);
    }

    private String dateToString(Date date) {
        if (date != null) {
            String pattern = "yyyyMMddHHmmss";
            DateFormat df = new SimpleDateFormat(pattern);
            return df.format(date);
        }
        return "";
    }
}
