package vn.com.seabank.biz.service;

import com.swib.seanet.topup.bean.*;
import com.swib.seanet.topup.service.TopUpApiClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import vn.com.seabank.biz.dto.ResponseObject;
import vn.com.seabank.biz.exception.CoreApiException;
import vn.com.seabank.biz.utils.BuildHeaderUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

import static vn.com.seabank.biz.config.Constants.BLANK;
import static vn.com.seabank.biz.config.Constants.PHONE_CARD_SERVICE_ID_VALUE;

/**
 * Created by sonba.itsol
 * Email: sonba.itsol@seabank.com.vn
 * Date: 3/24/2020 / 03 / 2020
 * Time: 9:30 AM
 */
@Service
public class PhoneCardService extends BuildHeaderUtils {

    private final Logger log = LoggerFactory.getLogger(PhoneCardService.class);

    /**
     * mua the cao
     *
     * @param bodyRequest
     * @return
     */
    public ResponseObject<TopUpResponse> buyPhoneCard(HttpServletRequest request, TopUpBodyRequest bodyRequest) throws CoreApiException {
        String serviceProviderId = bodyRequest.getServiceProviderId();
        bodyRequest.setServiceProviderId(convertProviderCode(serviceProviderId));
        bodyRequest.setPaymentCode(convertToPaymentCode(serviceProviderId));
        TopUpApiClient topUpApiClient = new TopUpApiClient();
        TopUpHeaderRequest topUpHeaderRequest = createTopUpHeader(request);
        bodyRequest.setTransRequestId(getTransId());
        if (StringUtils.isEmpty(bodyRequest.getAdditionalInfo())) {
            bodyRequest.setAdditionalInfo("MUA THE CAO");
        }
        bodyRequest.setOptions(BLANK);
        bodyRequest.setBillMonth(BLANK);
        bodyRequest.setUsername(getUsername(request));
        bodyRequest.setSessionId(getSessionId(request));
        TopUpResponse topUpResponse = topUpApiClient.topup(topUpHeaderRequest, bodyRequest, PHONE_CARD_SERVICE_ID_VALUE);
        return ResponseObject.success(topUpResponse);
    }

    /**
     * Truy vấn danh sách mã thẻ cào
     *
     * @param bodyRequest
     * @return
     */
    public ResponseObject<List<CardInfo>> getListCard(HttpServletRequest request, ListCardBodyRequest bodyRequest) throws CoreApiException {
        TopUpApiClient topUpApiClient = new TopUpApiClient();
        TopUpHeaderRequest topUpHeaderRequest = createTopUpHeader(request);
        bodyRequest.setUsername(getUsername(request));
        // TEST DATA
//        if ("1468963300".equals(getUsername(request))) {
//            List<CardInfo> cardInfos = new ArrayList<>();
//            for (int i = 0; i < 5; i++) {
//                int amount = (i+1) * 20000;
//                CardInfo card = new CardInfo();
//                card.setTraceid("005242");
//                card.setAction("RELIEF");
//                card.setPin_code("");
//                card.setProduct_code("");
//                card.setSerial("");
//                card.setService_id("37");
//                card.setService_provider_id("GAC");
//                card.setPhone_no("GA");
//                card.setFt("FT20004242940020");
//                card.setDebit_account("02200000172475");
//                card.setChannel("SEANET");
//                card.setBill_amount(String.valueOf(amount));
//                card.setProcess_date("20/03/2020");
//                cardInfos.add(card);
//            }
//            return ResponseObject.success(cardInfos);
//        }

        List<CardInfo> cardInfos = topUpApiClient.getListCard(topUpHeaderRequest, bodyRequest);
        return ResponseObject.success(cardInfos);
    }
}
