package vn.com.seabank.biz.service;

import com.seabank.biz.cardopcde.bean.*;
import com.seabank.biz.cardopcde.service.CardOPCDEAPIClient;
import com.seabank.biz.custinfo.bean.AccountInfo;
import com.seabank.biz.custinfo.bean.UserProfileResponse;
import com.seabank.biz.utility.bean.*;
import com.seabank.biz.utility.service.UtilityAPIClient;
import com.swib.seanet.t24enquiry.bean.PartnerServiceResponse;
import com.swib.seanet.t24enquiry.bean.T24Header;
import com.swib.seanet.t24enquiry.service.T24EnquiryApiClient;
import com.swib.seanet.topup.bean.*;
import com.swib.seanet.topup.service.TopUpApiClient;
import com.swib.services.config.CacheConfiguration;
import com.swib.services.service.SWIBEnquiryService;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import vn.com.seabank.biz.bean.VETCGetAmountParam;
import vn.com.seabank.biz.bean.VETCGetAmountResponse;
import vn.com.seabank.biz.bean.VETCTransactionParam;
import vn.com.seabank.biz.bean.VETCTransactionResponse;
import vn.com.seabank.biz.config.ConfigProperties;
import vn.com.seabank.biz.domain.*;
import vn.com.seabank.biz.dto.ResponseObject;
import vn.com.seabank.biz.exception.CoreApiException;
import vn.com.seabank.biz.exception.SwibApiException;
import vn.com.seabank.biz.ultil.ErrorMessage;
import vn.com.seabank.biz.utils.BuildHeaderUtils;

import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.IOException;
import java.io.StringReader;
import java.util.*;
import java.util.concurrent.TimeUnit;

import static vn.com.seabank.biz.config.Constants.*;

/**
 * Created by sonba.itsol
 * Email: sonba.itsol@seabank.com.vn
 * Date: 3/23/2020 / 03 / 2020
 * Time: 10:20 AM
 */

@Service
public class TopUpService extends BuildHeaderUtils {

    private final Logger log = LoggerFactory.getLogger(TopUpService.class);

    @Inject
    private SWIBEnquiryService swibEnquiryService;

    @Inject
    private ConfigProperties configProperties;

    private String VETC_SOAP_REQUEST = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:ser=\"http://payment.seabank.com.vn/service\"><soapenv:Header/><soapenv:Body><ser:process><!--Optional:--><request>%s</request></ser:process></soapenv:Body></soapenv:Envelope>";

    /**
     * Get list account for topup function
     *
     * @param packAge
     * @return
     * @throws SwibApiException
     */
    public ResponseObject<Object> getListAccountForTopUp(HttpServletRequest request, String packAge) throws Exception {
        List<AccountInfo> result = new ArrayList<>();
        Map<String, Object> response = new HashMap<>();
        List<AccountInfo> lstAccount = swibEnquiryService.listCustomerAccounts(getUserSessionInfo(request), true);
        List<FtCfgCache> ftCfgCaches = (List<FtCfgCache>) CacheConfiguration.getHazelcastInstance().getMap("ADMIN").get("TRANSFER_CACHE");
        String ftTypeId;
        List<FtRule> ftRules = new ArrayList<>();
        for (FtCfgCache ftCfgCache : ftCfgCaches) {
            FtCfg ftCfg = ftCfgCache.getFtCfg();
            ftTypeId = ftCfg.getFtTypeId().getFtTypeId();
            String spId = ftCfg.getSpId().getSpId();
            if (ftTypeId.equals(TOPUP) && spId.equals(packAge) && ftCfgCache.getFtRule() != null) {
                ftRules.addAll(ftCfgCache.getFtRule());
                FtCfg regexCfg = ftCfgCache.getFtCfg();
                regexCfg.getFtTypeId().setFtCfgs(null);
                regexCfg.getSpId().setFtCfgs(null);
                response.put("regexCfg", regexCfg);
            }
        }

        for (FtRule ftRule : ftRules) {
            AccGrp accGrp = ftRule.getFromAccGrp();
            StringBuilder tempAccExclusive = new StringBuilder();
            if (accGrp.getAccountExclusive() != null && !accGrp.getAccountExclusive().isEmpty()) {
                tempAccExclusive.append(accGrp.getAccountExclusive()).append(",");
            }

            String[] lstAccCat = (accGrp.getAccCat() + ",").split("\\,");
            List<String> lstCatDisplayAsChart = Arrays.asList(lstAccCat);

            String[] lstAccCurrency = (accGrp.getCurency() + ",").split("\\,");
            List<String> currency = Arrays.asList(lstAccCurrency);

            String[] lstAccExclusive = tempAccExclusive.toString().split("\\,");
            List<String> accExclusives = Arrays.asList(lstAccExclusive);

            result.addAll(checkAccList(lstAccount, lstCatDisplayAsChart, currency, accExclusives));
        }
        result = new ArrayList<>(new LinkedHashSet<>(result));
        response.put("accountList", result);
        return ResponseObject.success(response);
    }

    private List<AccountInfo> checkAccList(List<AccountInfo> lstAccount, List<String> lstCatDisplayAsChart, List<String> curency, List<String> accExclusives) {
        List<AccountInfo> result = new ArrayList<>();
        for (AccountInfo accountInfo : lstAccount) {
            if (trollAcc(lstCatDisplayAsChart, accountInfo) && trollCurrency(curency, accountInfo) && trollMethod(accExclusives, accountInfo)) {
                result.add(accountInfo);
            }
        }
        return result;
    }

    /**
     * Phuong thuc duoc tach ra de check getCategory
     *
     * @param lstCatDisplayAsCharts
     * @param accountInfo
     * @return
     */
    private Boolean trollAcc(List<String> lstCatDisplayAsCharts, AccountInfo accountInfo) {
        for (String lstCatDisplayAsChart : lstCatDisplayAsCharts) {
            if (!lstCatDisplayAsChart.equals("") && lstCatDisplayAsChart.trim().equals(accountInfo.getProductCode().trim())) {
                return true;
            }
        }
        return false;
    }

    /**
     * Phuong thuc duoc tach ra de check currencys
     *
     * @param currencys
     * @param accountInfo
     * @return
     */
    private Boolean trollCurrency(List<String> currencys, AccountInfo accountInfo) {
        for (String currency : currencys) {
            if (!currency.equals("") && currency.trim().equals(accountInfo.getCurrency().trim())) {
                return true;
            }
        }
        return false;
    }

    /**
     * Phuong thuc duoc tach ra de check tai khoan loai tru
     *
     * @param accExclusives
     * @param accountInfo
     * @return
     */
    private Boolean trollMethod(List<String> accExclusives, AccountInfo accountInfo) {
        for (String accExclusive : accExclusives) {
            if (!accExclusive.equals("") && accountInfo.getAccountID().indexOf(accExclusive) == 0) {
                return false;
            }
        }
        return true;
    }

    /**
     * Truy van danh sach cac nha cung cap cua mot dich vu
     *
     * @return
     * @throws SwibApiException
     */
    public ResponseObject<List<ProviderServiceInfo>> getProviderService(HttpServletRequest request, String serviceId) throws IOException {
        List<ProviderServiceInfo> transaction = (List<ProviderServiceInfo>) CacheConfiguration.getHazelcastInstance().getMap(TOPUP).get(TOPUP_PROVIDER_CACHE + serviceId);
        log.debug("RESPONSE request to get transaction getProviderService from Cache" + transaction);
        if (CollectionUtils.isEmpty(transaction)) {
            UtilityHeaderRequest headerRequest = createUtilityHeader(request);
            ProviderServiceBodyRequest bodyRequest = new ProviderServiceBodyRequest();
            bodyRequest.setServiceID(serviceId);
            bodyRequest.setAuthenType(GET_PROVIDER_SERVICE);
            UtilityAPIClient utilityAPIClient = new UtilityAPIClient();
            ProviderServiceResponse providerServiceResponse = utilityAPIClient.getProviderService(headerRequest, bodyRequest);
            log.debug("RESPONSE request to get transaction getProviderService" + providerServiceResponse);
            if (providerServiceResponse != null && providerServiceResponse.getResponseCode() != null && providerServiceResponse.getResponseCode().equals("00")) {
                transaction = providerServiceResponse.getProvider();
                CacheConfiguration.getHazelcastInstance().getMap(TOPUP).put(TOPUP_PROVIDER_CACHE + serviceId, transaction);
            }
        }
        List<ProviderServiceInfo> result = new ArrayList<>();
        if (!CollectionUtils.isEmpty(transaction)) {
            for (ProviderServiceInfo item : transaction) {
                if ("ACTIVE".equalsIgnoreCase(item.getStatus())) {
                    result.add(item);
                }
            }
        }
        return ResponseObject.success(result);
    }

    /**
     * getPartnerService - Truy Van Partner Service
     *
     * @param request
     * @param providerId
     * @return
     * @throws SwibApiException
     */
    public ResponseObject<PartnerServiceResponse> getPartnerTopUpService(HttpServletRequest request, String providerId) {
        T24Header header = createT24Header(request);
        T24EnquiryApiClient t24EnquiryApiClient = new T24EnquiryApiClient(header);
        PartnerServiceResponse partnerServiceResponse = t24EnquiryApiClient.getPartnerService("T24ENQ.EBANK.GET.PARTNER.SERVICES", providerId);
        return ResponseObject.success(partnerServiceResponse);
    }

    /**
     * TopUpTransaction
     *
     * @param bodyRequest
     * @return
     */
    public ResponseObject<TopUpTransactionResponse> topUpTransaction(HttpServletRequest request, TopUpTransactionBodyRequest bodyRequest) {
        TopUpTransactionResponse topUpTransactionResponse;
        String userName = getUsername(request);
        UserSessionInfo userSessionInfo = getUserSessionInfo(request);
        bodyRequest.setClientId(configProperties.getDpw().getHeader().getClientId());
        bodyRequest.setSecretId(configProperties.getDpw().getHeader().getSecret());
        bodyRequest.setUserName(userName);
        bodyRequest.setCustomerId(getCustomerId(request));
        bodyRequest.setSessionId(getSessionId(request));
        bodyRequest.setPassword(getPassword(request));
        bodyRequest.setUri(configProperties.getTopup().getHeader().getUri());
        bodyRequest.setChannel(userSessionInfo.getChannel());
        bodyRequest.setSubChannel(userSessionInfo.getSubChannel());
        bodyRequest.setFrom(userSessionInfo.getSubChannel());
        TopUpApiClient topUpApiClient = new TopUpApiClient();
        topUpTransactionResponse = topUpApiClient.TopUpTransaction(bodyRequest);

        // check discount remain limit credit card when transfer successful
//        if (!StringUtils.isEmpty(bodyRequest.getTokenCard())) {
//            updateLimitRemainCredit(bodyRequest.getTokenCard(), userName, bodyRequest.getTransAmount());
//        }
        return ResponseObject.success(topUpTransactionResponse);
    }

    /**
     * TopUpTransaction
     *
     * @param bodyRequest
     * @return
     */
    public ResponseObject<TopUpResponse> topUpTransactionByVNPT(HttpServletRequest request, TopUpBodyRequest bodyRequest) {
        TopUpResponse topUpResponse;
        String userName = getUsername(request);
        TopUpApiClient topUpApiClient = new TopUpApiClient();
        TopUpHeaderRequest topUpHeaderRequest = createTopUpHeader(request);
        bodyRequest.setTransRequestId(getTransId());
        bodyRequest.setServiceProviderId(convertProviderCode(bodyRequest.getServiceProviderId()));
        if (StringUtils.isEmpty(bodyRequest.getAdditionalInfo())) {
            bodyRequest.setAdditionalInfo("NAP THE DIEN THOAI");
        }
        bodyRequest.setOptions(BLANK);
        bodyRequest.setBillMonth(BLANK);
        bodyRequest.setUsername(userName);
        bodyRequest.setSessionId(getSessionId(request));
        topUpResponse = topUpApiClient.topup(topUpHeaderRequest, bodyRequest, TOPUP_SERVICE_ID_VALUE);

        // check discount remain limit credit card when transfer successful
//        if (!StringUtils.isEmpty(bodyRequest.getTokenCard())) {
//            updateLimitRemainCredit(bodyRequest.getTokenCard(), userName, bodyRequest.getBillAmount());
//        }
        return ResponseObject.success(topUpResponse);
    }

    // discount remain limit credit card when transfer successful
//    private void updateLimitRemainCredit(String tokenCard, String username, String amount) {
//        try {
//            List<Card> lstCardEfficiency = (List<Card>) CacheConfiguration.getHazelcastInstance().getMap(LIST_CARD_FULL).get(CUSTOMER_ + username);
//            log.info("lstCardEfficiency NULL: {}", lstCardEfficiency == null);
//            for (int i = 0; i < Objects.requireNonNull(lstCardEfficiency).size(); i++) {
//                if (tokenCard.equals(lstCardEfficiency.get(i).getTokenCard())) {
//                    // discount remain limit credit card when transfer successful
//                    Double limitRemain = Double.parseDouble(lstCardEfficiency.get(i).getLimitRemain()) - Double.parseDouble(amount);
//                    lstCardEfficiency.get(i).setLimitRemain(String.valueOf(limitRemain));
//                    CacheConfiguration.getHazelcastInstance().getMap(LIST_CARD_FULL).put(CUSTOMER_ + username, lstCardEfficiency, 30, TimeUnit.MINUTES);
//                    break;
//                }
//            }
//        } catch (Exception e) {
//            log.error("updateLimitRemainCredit: ", e);
//        }
//    }

    // discount remain limit credit card when transfer successful
//    private void revertLimitRemainCredit(String tokenCard, String username, String amount) {
//        try {
//            List<Card> lstCardEfficiency = (List<Card>) CacheConfiguration.getHazelcastInstance().getMap(LIST_CARD_FULL).get(CUSTOMER_ + username);
//            for (int i = 0; i < lstCardEfficiency.size(); i++) {
//                if (tokenCard.equals(lstCardEfficiency.get(i).getTokenCard())) {
//                    // discount remain limit credit card when transfer successful
//                    Double limitRemain = Double.parseDouble(lstCardEfficiency.get(i).getLimitRemain()) + Double.parseDouble(amount);
//                    lstCardEfficiency.get(i).setLimitRemain(String.valueOf(limitRemain));
//                    CacheConfiguration.getHazelcastInstance().getMap(LIST_CARD_FULL).put(CUSTOMER_ + username, lstCardEfficiency, 30, TimeUnit.MINUTES);
//                    break;
//                }
//            }
//        } catch (Exception e) {
//            log.error("revertLimitRemainCredit: ", e);
//        }
//    }

    /**
     * Get OTP Pay Card
     *
     * @param request
     * @param bodyRequest
     * @return
     * @throws Exception
     */
    public ResponseObject<GetAcqPurchaseOTPRes> getAcqPurchaseOTP(HttpServletRequest request, GetAcqPurchaseOTPReq bodyRequest) throws Exception {
        UserProfileResponse userProfileResponse = getUserProfile(request);
        bodyRequest.setPhone(userProfileResponse.getPhone());
        CardOPCDEHeaderRequest headerRequest = createCardOPCDEHeader(request);
        CardOPCDEAPIClient cardOPCDEAPIClient = new CardOPCDEAPIClient();
        return ResponseObject.success(cardOPCDEAPIClient.getAcqPurchaseOTP(headerRequest, bodyRequest));
    }

    public ResponseObject<GetPartnerAcqPurchaseRes> getPartnerAcqPurchase(HttpServletRequest request, GetPartnerAcqPurchaseReq bodyRequest) throws Exception {
        String pass = getPassword(request);
        if (bodyRequest.getPassword() == null || !bodyRequest.getPassword().equalsIgnoreCase(pass)) {
            throw new CoreApiException(ErrorMessage.PASSWORD_NOT_MATCH.getCode(), ErrorMessage.PASSWORD_NOT_MATCH.getMessage(), null);
        }
        bodyRequest.setSessionId(getSessionId(request));
        CardOPCDEHeaderRequest headerRequest = createCardOPCDEHeader(request);
        CardOPCDEAPIClient cardOPCDEAPIClient = new CardOPCDEAPIClient();
        return ResponseObject.success(cardOPCDEAPIClient.getPartnerAcqPurchase(headerRequest, bodyRequest));
    }

    /**
     * getListTopUpService - Truy van danh sach cac dich vu con cua dich vu BILLING
     *
     * @return
     */
    public ResponseObject<List<ServiceParam>> getListTopUpService(HttpServletRequest request) throws IOException {
        log.debug("REST request to get transaction getServiceChildren");
        List<ServiceInfo> serviceInfos = null;
        List<ServiceParam> serviceProvider = CacheConfiguration.getHazelcastInstance().getList("LIST_TOPUP_SERVICES_PROVIDER");
        if (CollectionUtils.isEmpty(serviceProvider) || serviceProvider.size() == 0) {
            log.debug("REST request to get serviveProvider get prepaidCard");
            UtilityHeaderRequest headerRequest = createUtilityHeader(request);
            ServiceBodyRequest bodyRequest = new ServiceBodyRequest();
            bodyRequest.setAuthenType(GET_SERVICES);
            bodyRequest.setEpayType(SERVICE);
            UtilityAPIClient utilityAPIClient = new UtilityAPIClient();
            ServiceResponse serviceResponse = utilityAPIClient.getSERVICES(headerRequest, bodyRequest);
            if (serviceResponse != null && serviceResponse.getResponseCode() != null && "00".equals(serviceResponse.getResponseCode())) {
                serviceInfos = serviceResponse.getService();
                for (ServiceInfo serviceInfo : serviceInfos) {
                    ServiceParam temp = new ServiceParam(serviceInfo);
                    serviceProvider.add(temp);
                }
                CacheConfiguration.getHazelcastInstance().getList("LIST_TOPUP_SERVICES_PROVIDER").add(serviceProvider);
            }
        }
        return ResponseObject.success(serviceProvider);
    }

    public ResponseObject<VETCGetAmountResponse> getVETCAmountFee(HttpServletRequest httpServletRequest, VETCGetAmountParam param) throws Exception {
        VETCGetAmountResponse responseRequest = new VETCGetAmountResponse();
        String request = createGetVETCAmountFeeRequest(param, "ENQUIRY", getUserSessionInfo(httpServletRequest));
        String getVETCAmountFeeRequest = String.format(VETC_SOAP_REQUEST, request);
        StringEntity stringEntity = new StringEntity(getVETCAmountFeeRequest, "UTF-8");
        stringEntity.setChunked(true);
        HttpPost httpPost = new HttpPost(configProperties.getVetc().getHeader().getUri());
        httpPost.setEntity(stringEntity);
        httpPost.addHeader("Content-Type", "text/xml; charset=utf-8");
        httpPost.addHeader("X-IBM-Client-Id", configProperties.getDpw().getHeader().getClientId());
        httpPost.addHeader("X-IBM-Client-Secret", configProperties.getDpw().getHeader().getSecret());

        if (mockGetVETCFEE(param.getCustomerCode()) != null) {
            return ResponseObject.success(mockGetVETCFEE(param.getCustomerCode()));
        }

        // Execute and get the response.
        HttpClient httpClient = new DefaultHttpClient();
        HttpResponse response = httpClient.execute(httpPost);
        org.apache.http.HttpEntity entity = response.getEntity();
        if (entity != null) {
            String strResponse = EntityUtils.toString(entity);
            strResponse = strResponse.replace("&lt;", "<");
            strResponse = strResponse.replace("&gt;", ">");
            DocumentBuilderFactory builderFactory = DocumentBuilderFactory.newInstance();
            builderFactory.setFeature("http://xml.org/sax/features/external-general-entities", false);
            builderFactory.setFeature("http://xml.org/sax/features/external-parameter-entities", false);
//            builderFactory.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
            DocumentBuilder builder = builderFactory.newDocumentBuilder();
            InputSource src = new InputSource();
            src.setCharacterStream(new StringReader(strResponse));
            Document doc = builder.parse(src);
            if (strResponse.contains("CustomerName")) {
                String customerName = doc.getElementsByTagName("CustomerName").item(0).getTextContent();
                String amountWithFee = doc.getElementsByTagName("AmountWithFee").item(0).getTextContent();
                responseRequest.setCustomerName(customerName);
                responseRequest.setTotalAmount(amountWithFee);
                return ResponseObject.success(responseRequest);
            } else {
                String[] responseError = doc.getElementsByTagName("response").item(0).getTextContent().split("/");
                String[] resErr = responseError[1].split("-");
                if (responseError[1].startsWith("-")) {
                    throw new CoreApiException(resErr[1], resErr[2], CoreApiException.PrefixErrorCode.TOPUPBILLING);
                } else {
                    throw new CoreApiException(resErr[0], resErr[1], CoreApiException.PrefixErrorCode.TOPUPBILLING);
                }
            }
        }
        throw new CoreApiException(ErrorMessage.INTERNAL_SERVER_ERROR.getCode(), ErrorMessage.INTERNAL_SERVER_ERROR.getMessage(), null);
    }

    public ResponseObject<VETCTransactionResponse> transactionVETCAmount(HttpServletRequest httpServletRequest, VETCTransactionParam param) throws Exception {
        VETCTransactionResponse responseRequest = new VETCTransactionResponse();
        String request = createTopupVETCAmountFeeRequest(param, "TRANSACTION", getUserSessionInfo(httpServletRequest));
        String transactionVETCAmountFeeRequest = String.format(VETC_SOAP_REQUEST, request);
        StringEntity stringEntity = new StringEntity(transactionVETCAmountFeeRequest, "UTF-8");
        log.debug("Seabank  transactionrequest : " + transactionVETCAmountFeeRequest);
        stringEntity.setChunked(true);
        HttpPost httpPost = new HttpPost(configProperties.getVetc().getHeader().getUri());
        httpPost.setEntity(stringEntity);
        httpPost.addHeader("Content-Type", "text/xml; charset=utf-8");
        httpPost.addHeader("X-IBM-Client-Id", configProperties.getDpw().getHeader().getClientId());
        httpPost.addHeader("X-IBM-Client-Secret", configProperties.getDpw().getHeader().getSecret());

        if ("e0100140443".equals(param.getCustomerCode())) {
            responseRequest.setFtTransaction("FT20015501281979");
            responseRequest.setResCode("");
            responseRequest.setResDesc("");
            return ResponseObject.success(responseRequest);
        }

        // Execute and get the response.
        HttpClient httpClient = new DefaultHttpClient();
        HttpResponse response = httpClient.execute(httpPost);
        org.apache.http.HttpEntity entity = response.getEntity();
        if (entity != null) {
            String strResponse = EntityUtils.toString(entity);
            strResponse = strResponse.replace("&lt;", "<");
            strResponse = strResponse.replace("&gt;", ">");
            DocumentBuilderFactory builderFactory = DocumentBuilderFactory.newInstance();
            builderFactory.setFeature("http://xml.org/sax/features/external-general-entities", false);
            builderFactory.setFeature("http://xml.org/sax/features/external-parameter-entities", false);
//            builderFactory.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
            DocumentBuilder builder = builderFactory.newDocumentBuilder();
            InputSource src = new InputSource();
            src.setCharacterStream(new StringReader(strResponse));
            Document doc = builder.parse(src);
            String responseTransaction = doc.getElementsByTagName("response").item(0).getTextContent();
            if (strResponse.contains("-1")) {
                String[] resErr = responseTransaction.split("-");
                throw new CoreApiException(resErr[2], resErr[resErr.length - 1], CoreApiException.PrefixErrorCode.TOPUPBILLING);
            } else {
                String[] resFt = responseTransaction.split("/");
                responseRequest.setFtTransaction(resFt[1]);
                return ResponseObject.success(responseRequest);
            }
        }
        throw new CoreApiException(ErrorMessage.INTERNAL_SERVER_ERROR.getCode(), ErrorMessage.INTERNAL_SERVER_ERROR.getMessage(), null);
    }

    public String createGetVETCAmountFeeRequest(VETCGetAmountParam param, String reqType, UserSessionInfo userSessionInfo) {
        StringBuilder gotadiHeaderRequest = new StringBuilder("<![CDATA[<Document>");
        gotadiHeaderRequest.append("<CustomerCode>");
        gotadiHeaderRequest.append(param.getCustomerCode());
        gotadiHeaderRequest.append("</CustomerCode>");
        gotadiHeaderRequest.append("<ProviderId>");
        gotadiHeaderRequest.append(param.getProviderId());
        gotadiHeaderRequest.append("</ProviderId>");
        gotadiHeaderRequest.append("<SubService>");
        gotadiHeaderRequest.append("SERVICE-VETC");
        gotadiHeaderRequest.append("</SubService>");
        gotadiHeaderRequest.append("<CompanyCode>");
        gotadiHeaderRequest.append(param.getCompany());
        gotadiHeaderRequest.append("</CompanyCode>");
        gotadiHeaderRequest.append("<CustomerId>");
        gotadiHeaderRequest.append(param.getCustomerId());
        gotadiHeaderRequest.append("</CustomerId>");
        gotadiHeaderRequest.append("<Password/>");
        gotadiHeaderRequest.append("<sessionid>");
        gotadiHeaderRequest.append("</sessionid>");
        gotadiHeaderRequest.append("<subChannel>");
        gotadiHeaderRequest.append(userSessionInfo.getSubChannel());
        gotadiHeaderRequest.append("</subChannel>");
        gotadiHeaderRequest.append("<UserName>");
        gotadiHeaderRequest.append(param.getUserId());
        gotadiHeaderRequest.append("</UserName>");
        gotadiHeaderRequest.append("<Channel>");
        gotadiHeaderRequest.append(userSessionInfo.getChannel());
//        gotadiHeaderRequest.append("SEAMOBILE");
        gotadiHeaderRequest.append("</Channel>");
        gotadiHeaderRequest.append("<CustomerAccount>");
        gotadiHeaderRequest.append(param.getCustomerAccount());
        gotadiHeaderRequest.append("</CustomerAccount>");
        gotadiHeaderRequest.append("<CustomerPIN/>");
        gotadiHeaderRequest.append("<from/>");
        gotadiHeaderRequest.append("<LanguageId>2</LanguageId>");
        gotadiHeaderRequest.append("<Partner/>");
        gotadiHeaderRequest.append("<RequestType>");
        gotadiHeaderRequest.append(reqType);
        gotadiHeaderRequest.append("</RequestType>");
        gotadiHeaderRequest.append("<Service>");
        gotadiHeaderRequest.append(BILLING);
        gotadiHeaderRequest.append("</Service>");
        gotadiHeaderRequest.append("<Bill>");
        gotadiHeaderRequest.append("<Amount>");
        gotadiHeaderRequest.append(param.getAmount());
        gotadiHeaderRequest.append("</Amount>");
        gotadiHeaderRequest.append("</Bill>");
        gotadiHeaderRequest.append("</Document>]]>");
        return gotadiHeaderRequest.toString();
    }

    public String createTopupVETCAmountFeeRequest(VETCTransactionParam param, String reqType, UserSessionInfo userSessionInfo) {
        StringBuilder gotadiHeaderRequest = new StringBuilder("<![CDATA[<Document>");
        gotadiHeaderRequest.append("<UserName>");
        gotadiHeaderRequest.append(param.getUserId());
        gotadiHeaderRequest.append("</UserName>");
        gotadiHeaderRequest.append("<Password>");
        gotadiHeaderRequest.append(param.getPassword());
        gotadiHeaderRequest.append("</Password>");
        gotadiHeaderRequest.append("<CustomerId>");
        gotadiHeaderRequest.append(param.getCustomerId());
        gotadiHeaderRequest.append("</CustomerId>");
        gotadiHeaderRequest.append("<SessionId>");
        gotadiHeaderRequest.append("0");
        gotadiHeaderRequest.append("</SessionId>");
        gotadiHeaderRequest.append("<Service>");
        gotadiHeaderRequest.append(BILLING);
        gotadiHeaderRequest.append("</Service>");
        gotadiHeaderRequest.append("<SubService>");
        gotadiHeaderRequest.append("SERVICE-VETC");
        gotadiHeaderRequest.append("</SubService>");
        gotadiHeaderRequest.append("<CustomerPhone>");
        gotadiHeaderRequest.append("</CustomerPhone>");
        gotadiHeaderRequest.append("<Partner>");
        gotadiHeaderRequest.append("VNPAY");
        gotadiHeaderRequest.append("</Partner>");
        gotadiHeaderRequest.append("<CustomerAccount>");
        gotadiHeaderRequest.append(param.getCustomerAccount());
        gotadiHeaderRequest.append("</CustomerAccount>");
        gotadiHeaderRequest.append("<CustomerPIN>");
        gotadiHeaderRequest.append("</CustomerPIN>");
        gotadiHeaderRequest.append("<CustomerCode>");
        gotadiHeaderRequest.append(param.getCustomerCode());
        gotadiHeaderRequest.append("</CustomerCode>");
        gotadiHeaderRequest.append("<CompanyCode>");
        gotadiHeaderRequest.append(param.getCompany());
        gotadiHeaderRequest.append("</CompanyCode>");
        gotadiHeaderRequest.append("<RequestType>");
        gotadiHeaderRequest.append(reqType);
        gotadiHeaderRequest.append("</RequestType>");
        gotadiHeaderRequest.append("<SubChannel>");
        gotadiHeaderRequest.append(userSessionInfo.getSubChannel());
        gotadiHeaderRequest.append("</SubChannel>");
        gotadiHeaderRequest.append("<Channel>");
        gotadiHeaderRequest.append(userSessionInfo.getChannel());
        gotadiHeaderRequest.append("</Channel>");
        gotadiHeaderRequest.append("<LanguageId>2</LanguageId>");
        gotadiHeaderRequest.append("<PDetail>");
        gotadiHeaderRequest.append(param.getDesc());
        gotadiHeaderRequest.append("</PDetail>");
        gotadiHeaderRequest.append("<ProviderId>");
        gotadiHeaderRequest.append(param.getProviderId());
        gotadiHeaderRequest.append("</ProviderId>");
        gotadiHeaderRequest.append("<Bill>");
        gotadiHeaderRequest.append("<Amount>");
        gotadiHeaderRequest.append(param.getTotalAmount());
        gotadiHeaderRequest.append("</Amount>");
        gotadiHeaderRequest.append("<OriginAmount>");
        gotadiHeaderRequest.append(param.getAmount());
        gotadiHeaderRequest.append("</OriginAmount>");
        gotadiHeaderRequest.append("</Bill>");
        gotadiHeaderRequest.append("</Document>]]>");
        return gotadiHeaderRequest.toString();
    }
}
