package vn.com.seabank.biz.utils;

import com.seabank.biz.authen.bean.AccountHeader;
import com.seabank.biz.authen.bean.BodyConfirmOTP;
import com.seabank.biz.authen.bean.ConfirmOTPResponse;
import com.seabank.biz.cardopcde.bean.CardOPCDEHeaderRequest;
import com.seabank.biz.custinfo.bean.UserProfileResponse;
import com.seabank.biz.utility.bean.UtilityHeaderRequest;
import com.swib.core.sso.casclient.userdetails.CasToken;
import com.swib.seanet.ben.bean.BenHeaderRequest;
import com.swib.seanet.t24enquiry.bean.T24Header;
import com.swib.seanet.topup.bean.TopUpHeaderRequest;
import com.swib.seanet.topup.bean.TopUpResponse;
import com.swib.seanet.topup.bean.TopUpTransactionResponse;
import com.swib.services.config.ApiClientProperties;
import com.swib.services.config.CacheConfiguration;
import com.swib.services.service.SWIBEnquiryService;
import com.swib.services.service.SwibCommonService;
import com.swib.services.utils.SwibUtils;
import com.swib.services.validators.CurrentAccountValidator;
import com.swib.services.validators.PersonalValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import vn.com.seabank.biz.bean.VETCGetAmountResponse;
import vn.com.seabank.biz.config.ConfigProperties;
import vn.com.seabank.biz.domain.UserSessionInfo;
import vn.com.seabank.biz.dto.MockFailedDTO;
import vn.com.seabank.biz.dto.ResponseObject;
import vn.com.seabank.biz.exception.CoreApiException;
import vn.com.seabank.biz.exception.SwibApiException;

import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static vn.com.seabank.biz.config.Constants.MAT_KHAU;

/**
 * Created by sonba.itsol
 * Email: sonba.itsol@seabank.com.vn
 * Date: 3/23/2020 / 03 / 2020
 * Time: 9:37 AM
 */
@Component
public class BuildHeaderUtils {

    @Inject
    private SwibUtils swibUtils;

    @Inject
    private ConfigProperties configProperties;

    @Inject
    private ApiClientProperties apiClientProperties;

    @Inject
    private SWIBEnquiryService swibEnquiryService;

    @Inject
    private SwibCommonService swibCommonService;

    @Autowired
    private CurrentAccountValidator currentAccountValidator;

    public UserSessionInfo getUserSessionInfo(HttpServletRequest request) {
        return swibUtils.getUserSessionInfo(request);
    }

    public String getSessionId(HttpServletRequest request) {
        return swibUtils.getSessionId(request);
    }

    public String getTransId() {
        return java.util.UUID.randomUUID().toString().replace("-", "").toUpperCase();
    }

    public String getCustomerId(HttpServletRequest request) {
        return swibUtils.getCustomerId(request);
    }

    public String getUsername(HttpServletRequest request) {
        return swibUtils.getUserName(request);
    }

    public UserProfileResponse getUserProfile(HttpServletRequest request) {
        return swibEnquiryService.getUserProfile(request);
    }

    public String getPassword(HttpServletRequest request) {
        String jwt = SwibUtils.resolveToken(request);
        CasToken casToken = swibUtils.getCasToken(jwt);
        return (String) CacheConfiguration.getHazelcastInstance().getMap(MAT_KHAU).get(casToken.getTgt());
    }

    // create header for api t24 enquiry
    public T24Header createT24Header(HttpServletRequest request) {
        UserSessionInfo userSessionInfo = getUserSessionInfo(request);
        T24Header header = new T24Header();
        header.setxIBMClientId(this.apiClientProperties.getDpw().getHeader().getClientId());
        header.setxIBMClientSecret(this.apiClientProperties.getDpw().getHeader().getSecret());
        header.setReqType(this.apiClientProperties.getCommon().getHeader().getReqType());
        header.setTrusted(this.apiClientProperties.getCommon().getHeader().getTrusted());
        header.setPriority(this.apiClientProperties.getAccount().getHeader().getPriority());
        header.setApi(this.apiClientProperties.getAccount().getHeader().getApi());
        header.setApiKey(this.apiClientProperties.getAccount().getHeader().getApiKey());
        header.setUri(this.apiClientProperties.getAccount().getHeader().getUri());
        header.setChannel(userSessionInfo.getChannel());
        header.setSubChannel(userSessionInfo.getSubChannel());
        header.setLocation(userSessionInfo.getLocation());
        header.setContext(userSessionInfo.getContext());
        header.setUserID(swibUtils.getUserName(request));
        header.setRequestAPI(swibUtils.getHostName(request));
        header.setRequestNode("01");
        return header;
    }

    /**
     * Create header request for TopUp API
     *
     * @return
     */
    public TopUpHeaderRequest createTopUpHeader(HttpServletRequest request) {
        UserSessionInfo userSessionInfo = getUserSessionInfo(request);
        TopUpHeaderRequest topUpHeaderRequest = new TopUpHeaderRequest();

        //DPW
        ConfigProperties.Dpw dpw = configProperties.getDpw();
        ConfigProperties.DpwHeader dpwHeader = dpw.getHeader();

        topUpHeaderRequest.setxIBMClientId(dpwHeader.getClientId());
        topUpHeaderRequest.setxIBMClientSecret(dpwHeader.getSecret());

        //TopUP API header
        ConfigProperties.TopUpVNPT topUpVNPT = configProperties.getTopUpVNPT();
        ConfigProperties.APIClientHeader apiClientHeader = topUpVNPT.getHeader();

        topUpHeaderRequest.setReqType(apiClientHeader.getReqType());
        topUpHeaderRequest.setTrusted(apiClientHeader.getTrusted());
        topUpHeaderRequest.setApi(apiClientHeader.getApi());
        topUpHeaderRequest.setApiKey(apiClientHeader.getApiKey());
        topUpHeaderRequest.setPriority(apiClientHeader.getPriority());
        topUpHeaderRequest.setUri(apiClientHeader.getUri());
        topUpHeaderRequest.setChannel(userSessionInfo.getChannel());
        topUpHeaderRequest.setSubChannel(userSessionInfo.getSubChannel());
        topUpHeaderRequest.setLocation(userSessionInfo.getLocation());
        topUpHeaderRequest.setContext(userSessionInfo.getContext());
        topUpHeaderRequest.setUserID(swibUtils.getUserName(request));
        topUpHeaderRequest.setRequestAPI(swibUtils.getHostName(request));
        topUpHeaderRequest.setRequestNode("01");
        topUpHeaderRequest.setTraceId(getTransId());
        return topUpHeaderRequest;
    }

    /**
     * Create header request for BenAPI
     *
     * @return
     */
    public BenHeaderRequest createBenHeader(HttpServletRequest request) {
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        BenHeaderRequest benHeaderRequest = new BenHeaderRequest();

        //DPW
        ConfigProperties.Dpw dpw = configProperties.getDpw();
        ConfigProperties.DpwHeader dpwHeader = dpw.getHeader();

        benHeaderRequest.setxIBMClientId(dpwHeader.getClientId());
        benHeaderRequest.setxIBMClientSecret(dpwHeader.getSecret());

        //ben API header
        ConfigProperties.Ben ben = configProperties.getBen();
        ConfigProperties.APIClientHeader apiClientHeader = ben.getHeader();

        benHeaderRequest.setReqType(apiClientHeader.getReqType());
        benHeaderRequest.setTrusted(apiClientHeader.getTrusted());
        benHeaderRequest.setApi(apiClientHeader.getApi());
        benHeaderRequest.setApiKey(apiClientHeader.getApiKey());
        benHeaderRequest.setPriority(apiClientHeader.getPriority());
        benHeaderRequest.setUri(apiClientHeader.getUri());


        benHeaderRequest.setChannel(userSessionInfo.getChannel());
        benHeaderRequest.setSubChannel(userSessionInfo.getSubChannel());
        benHeaderRequest.setLocation(userSessionInfo.getLocation());
        benHeaderRequest.setContext(userSessionInfo.getContext());
        benHeaderRequest.setUserID(swibUtils.getUserName(request));
        benHeaderRequest.setRequestAPI(swibUtils.getHostName(request));
        benHeaderRequest.setRequestNode("01");

        return benHeaderRequest;
    }

    /**
     * createAuthenHeader - Create header request for Authen API
     *
     * @return
     */
    public AccountHeader createAuthenHeader(HttpServletRequest request) {
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        AccountHeader accountHeader = new AccountHeader();

        //DPW
        ConfigProperties.Dpw dpw = configProperties.getDpw();
        ConfigProperties.DpwHeader dpwHeader = dpw.getHeader();

        accountHeader.setxIBMClientId(dpwHeader.getClientId());
        accountHeader.setxIBMClientSecret(dpwHeader.getSecret());

        //CusInfo API header
        ConfigProperties.Authen authen = configProperties.getAuthen();
        ConfigProperties.APIClientHeader apiClientHeader = authen.getHeader();

        accountHeader.setReqType(apiClientHeader.getReqType());
        accountHeader.setTrusted(apiClientHeader.getTrusted());
        accountHeader.setApi(apiClientHeader.getApi());
        accountHeader.setApiKey(apiClientHeader.getApiKey());
        accountHeader.setPriority(apiClientHeader.getPriority());
        accountHeader.setUri(apiClientHeader.getUri());

        accountHeader.setChannel(userSessionInfo.getChannel());
        accountHeader.setSubChannel(userSessionInfo.getSubChannel());
        accountHeader.setLocation(userSessionInfo.getLocation());
        accountHeader.setContext(userSessionInfo.getContext());
        accountHeader.setUserID(swibUtils.getUserName(request));
        accountHeader.setRequestAPI(swibUtils.getHostName(request));
        accountHeader.setRequestNode("01");

        return accountHeader;
    }

    /**
     * Create header request for UtilityAPI
     *
     * @return
     */
    public UtilityHeaderRequest createUtilityHeader(HttpServletRequest request) {
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        UtilityHeaderRequest header = new UtilityHeaderRequest();

        //DPW
        ConfigProperties.Dpw dpw = configProperties.getDpw();
        ConfigProperties.DpwHeader dpwHeader = dpw.getHeader();

        header.setxIBMClientId(dpwHeader.getClientId());
        header.setxIBMClientSecret(dpwHeader.getSecret());

        //utility API header
        ConfigProperties.Utility utility = configProperties.getUtility();
        ConfigProperties.APIClientHeader apiClientHeader = utility.getHeader();

        header.setReqType(apiClientHeader.getReqType());
        header.setTrusted(apiClientHeader.getTrusted());
        header.setApi(apiClientHeader.getApi());
        header.setApiKey(apiClientHeader.getApiKey());
        header.setPriority(apiClientHeader.getPriority());
        header.setUri(apiClientHeader.getUri());

        header.setChannel(userSessionInfo.getChannel());
        header.setSubChannel(userSessionInfo.getSubChannel());
        header.setLocation(userSessionInfo.getLocation());
        header.setContext(userSessionInfo.getContext());
        try {
            header.setUserID(swibUtils.getUserName(request));
        } catch (Exception e) {
            header.setUserID(apiClientHeader.getUserID());
        }
        header.setRequestAPI(swibUtils.getHostName(request));
        header.setRequestNode("01");

        return header;
    }

    /**
     * Create header request for CardOPCDE API
     *
     * @return
     */
    public CardOPCDEHeaderRequest createCardOPCDEHeader(HttpServletRequest request) {
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        CardOPCDEHeaderRequest header = new CardOPCDEHeaderRequest();

        //DPW
        ConfigProperties.Dpw dpw = configProperties.getDpw();
        ConfigProperties.DpwHeader dpwHeader = dpw.getHeader();

        header.setxIBMClientId(dpwHeader.getClientId());
        header.setxIBMClientSecret(dpwHeader.getSecret());

        //utility API header
        ConfigProperties.CardOPCDE cardOPCDE = configProperties.getCardOPCDE();
        ConfigProperties.APIClientHeader apiClientHeader = cardOPCDE.getHeader();

        header.setReqType(apiClientHeader.getReqType());
        header.setTrusted(apiClientHeader.getTrusted());
        header.setApi(apiClientHeader.getApi());
        header.setApiKey(apiClientHeader.getApiKey());
        header.setPriority(apiClientHeader.getPriority());
        header.setUri(apiClientHeader.getUri());

        header.setChannel(userSessionInfo.getChannel());
        header.setSubChannel(userSessionInfo.getSubChannel());
        header.setLocation(userSessionInfo.getLocation());
        header.setContext(userSessionInfo.getContext());
        header.setUserID(swibUtils.getUserName(request));
        header.setRequestAPI(swibUtils.getHostName(request));
        header.setRequestNode("01");

        return header;
    }

    public void checkAccountValid(HttpServletRequest request, String accountNumber) throws SwibApiException {
        PersonalValidator.Result rs = currentAccountValidator.validate((CurrentAccountValidator.ICurrentAccountValidator) () -> {
            List<String> accounts = new ArrayList<>();
            if (accountNumber != null) {
                accounts.add(accountNumber);
            }
            return accounts;
        }, request);
        if (!rs.isSuccess()) {
            throw new SwibApiException("Account Invalid!");
        }
    }

    public void checkAccountValid(HttpServletRequest request, Object bodyRequest) {
        PersonalValidator.Result rs = currentAccountValidator.validate(bodyRequest, request);
        if (!rs.isSuccess()) {
            throw new SwibApiException("Account Invalid!");
        }
    }

    public String convertProviderCode(String code) {
        if (!StringUtils.isEmpty(code)) {
            switch (code) {
                case "VINAFONE":
                    return "VNP";
                case "MOBIFONE":
                    return "MBP";
                case "VIETTEL":
                    return "VTL";
                case "EVNMOBILE":
                    return "BLE";
                case "VNMOBILE":
                    return "VNM";
                case "VCN":
                    return "VCN";
                case "VNG":
                    return "VNG";
                case "GCD":
                    return "GCD";
                case "OCH":
                    return "OCH";
                case "GAC":
                    return "GAC";
                default:
                    return null;
            }
        }
        return null;
    }

    public String convertToPaymentCode(String code) {
        switch (code) {
            case "VINAFONE":
                return "VP";
            case "MOBIFONE":
                return "MB";
            case "VIETTEL":
                return "VT";
            case "VCN":
                return "VC";
            case "VNG":
                return "VG";
            case "GCD":
                return "GC";
            case "OCH":
                return "OC";
            case "GAC":
                return "GA";
            default:
                return null;
        }
    }

    public void confirmOTP(HttpServletRequest request, String otp) {
        //Verify OTP
        BodyConfirmOTP bodyConfirmOTP = new BodyConfirmOTP();
        bodyConfirmOTP.setTransTime(String.valueOf(System.currentTimeMillis()));
        bodyConfirmOTP.setPassword(getPassword(request));
        bodyConfirmOTP.setTransInfo(getCustomerId(request) + "*topup-transaction");
        bodyConfirmOTP.setValidateCode(otp);
        ConfirmOTPResponse confirmOTPResponse = swibCommonService.confirmOTP(request, bodyConfirmOTP);
        if (!"00".equalsIgnoreCase(confirmOTPResponse.getResponseCode())) {
            throw new CoreApiException(confirmOTPResponse.getErrorCode(), confirmOTPResponse.getErrorMessage(), CoreApiException.PrefixErrorCode.AUTHEN);
        }
    }

    public ResponseObject<TopUpTransactionResponse> mockResponseTopUpGW(String paymentCode) {
        /*if (paymentCode.equalsIgnoreCase("012345678") || paymentCode.equalsIgnoreCase("0973464504")) {
            TopUpTransactionResponse resMock = new TopUpTransactionResponse();
            resMock.setResponseCode("1");
            resMock.setTransactionNO("FT123456789");
            return ResponseObject.success(resMock);
        }*/
        return null;
    }

    public ResponseObject<TopUpResponse> mockResponseTopUpVNPT(String paymentCode) {
        /*if (paymentCode.equalsIgnoreCase("0123456789") || paymentCode.equalsIgnoreCase("0973464504")) {
            TopUpResponse resMock = new TopUpResponse();
            resMock.setSecure_code("c5b38249d7a61c414f52a69f8dfc372a61766f2753d92d1d54c1764929bbe866");
            resMock.setDescription("Transaction done");
            resMock.setResponse_code("00");
            resMock.setTrans_date_time("20200413103203");
            resMock.setTrans_request_id("005472");
            resMock.setTrans_response_id("FT20013908490651");
            return ResponseObject.success(resMock);
        }

        Map<String, MockFailedDTO> failed = new HashMap<>();
        failed.put("0111111002", new MockFailedDTO("02", "Mã giao dịch đã tồn tại"));
        failed.put("0111111003", new MockFailedDTO("03", "Dữ liệu sai định dạng"));
        failed.put("0111111004", new MockFailedDTO("04", "Tài khoản đối tác không tồn tại"));
        failed.put("0111111005", new MockFailedDTO("05", "Tài khoản đối tác bị khóa"));
        failed.put("0111111006", new MockFailedDTO("06", "Chữ ký không hơp lệ"));
        failed.put("0111111007", new MockFailedDTO("07", "Lỗi hệ thống"));
        failed.put("0111111008", new MockFailedDTO("08", "Không tìm thấy nợ cước"));
        failed.put("0111111009", new MockFailedDTO("09", "Mã khách hàng/mã thanh toán không hợp lệ"));
        failed.put("0111111021", new MockFailedDTO("21", "Mã tỉnh không đúng"));
        failed.put("0111111096", new MockFailedDTO("96", "Hệ thống đang bảo trì"));
        failed.put("0111111098", new MockFailedDTO("98", "Timeout"));
        failed.put("0111111020", new MockFailedDTO("20", "Tài khoản không có hiệu lực hoặc hết airtime"));
        failed.put("0111111099", new MockFailedDTO("99", "Các lỗi khác"));

        if (failed.get(paymentCode) != null) {
            throw new CoreApiException(failed.get(paymentCode).getCode(), failed.get(paymentCode).getDesc(), CoreApiException.PrefixErrorCode.TOPUPVNPT);
        }*/
        return null;
    }

    public VETCGetAmountResponse mockGetVETCFEE(String customerCode) {
        /*VETCGetAmountResponse responseRequest = new VETCGetAmountResponse();
        if ("e0100140443".equals(customerCode)) {
            responseRequest.setCustomerName("CHI NHANH CONG TY CO PHAN VAN TAI TOC HANH MAI LINH TAI DAK LAK");
            responseRequest.setTotalAmount("1003300");
            responseRequest.setResCode("");
            responseRequest.setResDesc("");
            return responseRequest;
        }*/
        return null;
    }
}
