package vn.com.seabank.biz.web.rest;

import com.codahale.metrics.annotation.Timed;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.seabank.biz.custinfo.bean.UserProfileResponse;
import com.seabank.biz.utility.bean.ProviderServiceInfo;
import com.swib.seanet.t24enquiry.bean.SaleAccountInfo;
import com.swib.seanet.t24enquiry.bean.T24Header;
import com.swib.seanet.t24enquiry.service.T24EnquiryApiClient;
import com.swib.seanet.topup.bean.CardInfo;
import com.swib.seanet.topup.bean.ListCardBodyRequest;
import com.swib.seanet.topup.bean.TopUpBodyRequest;
import com.swib.seanet.topup.bean.TopUpResponse;
import com.swib.services.config.CacheConfiguration;
import com.swib.services.domain.VoucherBatch;
import com.swib.services.service.CustomerObserver;
import com.swib.services.service.VoucherBatchService;
import com.swib.services.service.dto.UseVoucherRequestDTO;
import com.swib.services.service.dto.VoucherUsedDTO;
import com.swib.services.utils.SwibUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import vn.com.seabank.biz.config.CryptLib;
import vn.com.seabank.biz.domain.UserSessionInfo;
import vn.com.seabank.biz.dto.ResponseObject;
import vn.com.seabank.biz.exception.CoreApiException;
import vn.com.seabank.biz.service.PhoneCardService;
import vn.com.seabank.biz.service.TopUpService;
import vn.com.seabank.biz.utils.BuildHeaderUtils;

import javax.crypto.NoSuchPaddingException;
import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static vn.com.seabank.biz.config.Constants.*;

/**
 * Created by sonba.itsol
 * Email: sonba.itsol@seabank.com.vn
 * Date: 3/24/2020 / 03 / 2020
 * Time: 9:30 AM
 */
@RestController
@RequestMapping(value = "/api/p0603-phone-card")
public class PhoneCardResource extends BuildHeaderUtils {

    private final Logger log = LoggerFactory.getLogger(PhoneCardResource.class);

    private static final String VND = "VND";

    @Inject
    private TopUpService topUpService;

    @Inject
    private PhoneCardService phoneCardService;

    @Autowired
    VoucherBatchService voucherBatchService;

    @Inject
    private SwibUtils swibUtils;

    @Inject
    private CustomerObserver customerObserver;

    private static final String PHONE_CARD = "PHONE_CARD";
    private static final String CODE02 = "02";
    private static final String BANKAPI_ADMIN_VOUCHER = "BANKAPI-ADMIN-VOUCHER";

    CryptLib cryptLib;

    {
        try {
            cryptLib = new CryptLib();
        } catch (NoSuchAlgorithmException | NoSuchPaddingException e) {
            log.error("error khoi tao cryptLib: " + e.getMessage());
        }
    }

    /**
     * getAccountInfo - Lay danh sach tai khoan thanh toan theo cau hinh
     *
     * @param request
     * @return
     */
    @RequestMapping(value = "/get-account-info", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<Object>> getAccountInfoPhoneCard(HttpServletRequest request) throws Exception {
        UserProfileResponse userProfileResponse = getUserProfile(request);
        String key = userProfileResponse.getPackAge();
        ResponseObject<Object> transaction = topUpService.getListAccountForTopUp(request, key);
        return new ResponseEntity<>(transaction, HttpStatus.OK);
    }

    /**
     * Truy van danh sach nha cung cap cua nap the dien thoai
     *
     * @param request
     * @return Map
     */
    @RequestMapping(value = "/get-provider-phone-card", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<List<ProviderServiceInfo>>> getServiceProvider(HttpServletRequest request) throws IOException {
        ResponseObject<List<ProviderServiceInfo>> providerService = topUpService.getProviderService(request, SERVICE_CARD_ID);
        return new ResponseEntity<>(providerService, HttpStatus.OK);
    }

    /**
     * Truy van danh sach menh gia mua the cao
     *
     * @return Map
     */
    @RequestMapping(value = "/get-list-amount-phone-card", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<Object>> getListAmountPhoneCard() {
        Object lstAmountPhoneCard = CacheConfiguration.getHazelcastInstance().getMap(ADMIN).get(LIST_AMOUNT_PHONE_CARD_CACHE);
        if (lstAmountPhoneCard == null) {
            ObjectMapper objectMapper = new ObjectMapper();
            objectMapper.configure(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES, true);
            try {
                Object result = objectMapper.readValue(LIST_AMOUNT_PHONE_CARD, Object.class);
                CacheConfiguration.getHazelcastInstance().getMap(ADMIN).put(LIST_AMOUNT_PHONE_CARD_CACHE, result);
                return new ResponseEntity<>(ResponseObject.success(result), HttpStatus.OK);
            } catch (IOException e) {
                Map<String, Object> map = new HashMap<>();
                map.put("DEFAULT", new long[]{10000, 20000, 30000, 50000, 100000, 200000, 300000, 500000});
                return new ResponseEntity<>(ResponseObject.success(map), HttpStatus.OK);
            }
        }
        return new ResponseEntity<>(ResponseObject.success(lstAmountPhoneCard), HttpStatus.OK);
    }

    /**
     * phone card by VNPT
     *
     * @param request
     * @return
     */
    @RequestMapping(value = "/vnpt-phone-card-transaction", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<TopUpResponse>> phoneCardByVNPT(HttpServletRequest request, @RequestBody TopUpBodyRequest topUpBodyRequest) throws Exception {
        log.info("-----> START VNPT PHONE CARD:: ");
        VoucherBatch validVoucher = null;
        //check voucher information
        if (!StringUtils.isEmpty(topUpBodyRequest.getRegIssuanceName())) {
            validVoucher = voucherBatchService.getValidVoucher(request, topUpBodyRequest.getRegIssuanceName(), PHONE_CARD);
            if (validVoucher == null) {
                throw new CoreApiException(CODE02, "EVoucherCode is not exist or wrong type", BANKAPI_ADMIN_VOUCHER);
            }
        }
        //Verify OTP
        confirmOTP(request, topUpBodyRequest.getOtp());
        if (topUpBodyRequest.getServiceProviderId() != null && "VIETTELXXXXXXX".equals(topUpBodyRequest.getServiceProviderId())) {
            TopUpResponse responseTest = new TopUpResponse();
            if ("200000".equals(topUpBodyRequest.getBillAmount())) {
                responseTest.setSecure_code("44849ddd7b82ba2a1f1100442bb5d20e3edcc915e2ae01a82dd46e5a7d126ab5");
                responseTest.setDescription("Transaction done");
                responseTest.setPin_code("121314");
                responseTest.setProduct_code("");
                responseTest.setResponse_code("00");
                responseTest.setSerial("123456789");
                responseTest.setTrans_date_time("20200302092242");
                responseTest.setTrans_request_id("005232");
                responseTest.setTrans_response_id("FT20002066414525");
                return new ResponseEntity<>(ResponseObject.success(responseTest), HttpStatus.OK);
            } else {
                responseTest.setSecure_code("c1f1f7d1c907342a88de96185e8bbdc140986772641d1aef1b22ace1a3916057");
                responseTest.setDescription("Transaction done");
                responseTest.setPin_code("");
                responseTest.setProduct_code("");
                responseTest.setResponse_code("07");
                responseTest.setSerial("");
                responseTest.setTrans_date_time("20200302092242");
                responseTest.setTrans_request_id("005232");
                responseTest.setTrans_response_id("FT20013569742270");
                return new ResponseEntity<>(ResponseObject.error("BANKAPI-TOPUPBILLINGAPI-07", "Transaction done",
                    "Lỗi trong quá trình xử lý. Quý khách vui lòng thực hiện lại giao dịch. Xin lỗi quý khách vì sự bất tiện này.",
                    "Error in processing. Please try it again. Sorry for this inconvenience.", responseTest), HttpStatus.OK);
            }
        } else {

            // check topup by credit card or account
            if (StringUtils.isEmpty(topUpBodyRequest.getTokenCard())) {
                // Kiem tra tai khoan lay sao ke phai nam
                // trong danh sach tai khoan cua nguoi dung
                checkAccountValid(request, topUpBodyRequest.getDebitAccount());
            } else {
                // get sale account when topup by credit card
                T24Header header = createT24Header(request);
                T24EnquiryApiClient t24EnquiryApiClient = new T24EnquiryApiClient(header);
                SaleAccountInfo saleAccountInfo = t24EnquiryApiClient.getSaleAccByTokenCard(topUpBodyRequest.getTokenCard());
                topUpBodyRequest.setDebitAccount(saleAccountInfo.getAccountId());
            }

            ResponseObject<TopUpResponse> transaction = phoneCardService.buyPhoneCard(request, topUpBodyRequest);
            customerObserver.onAccountChanged(swibUtils.getUserSessionInfo(request));
            // decrypt serial
            if (!StringUtils.isEmpty(transaction.getData().getSerial())) {
                transaction.getData().setSerial(cryptLib.decrypt(transaction.getData().getSerial(), keyEncrytData, IvAESSecretKey));
            }
            // decrypt pin code
            if (!StringUtils.isEmpty(transaction.getData().getPin_code())) {
                transaction.getData().setPin_code(cryptLib.decrypt(transaction.getData().getPin_code(), keyEncrytData, IvAESSecretKey));
            }
            try {
                if (validVoucher != null) {
                    UseVoucherRequestDTO useVoucherRequestDTO = new UseVoucherRequestDTO();
                    useVoucherRequestDTO.setAccountNo(topUpBodyRequest.getDebitAccount());
                    useVoucherRequestDTO.setCreditCcy(VND);
                    useVoucherRequestDTO.setDebitCcy(VND);
                    useVoucherRequestDTO.setEvoucherCode(validVoucher.getEvoucherCode());
                    useVoucherRequestDTO.setTransId(transaction.getData().getTrans_response_id());
                    useVoucherRequestDTO.setTransAmt(topUpBodyRequest.getBillAmount());
                    useVoucherRequestDTO.setTransTypeReq(PHONE_CARD);
                    voucherBatchService.usedVoucher(request, useVoucherRequestDTO);
                }
            } catch (Exception ex) {
                log.error("-----> VNPT PHONE CARD ERROR:: {}", ex.getMessage());
            }
            transaction.getData().setCurrency(VND);
            log.info("-----> VNPT PHONE CARD SUCCESS:: " + transaction.getData().getTrans_response_id());
            return new ResponseEntity<>(transaction, HttpStatus.OK);
        }
    }

    /**
     * Truy vấn danh sách mã thẻ cào
     *
     * @param request
     * @return Map
     */
    @RequestMapping(value = "/get-list-phone-card", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<List<CardInfo>>> getCardList(HttpServletRequest request, @RequestBody ListCardBodyRequest bodyRequest) throws Exception {
        ResponseObject<List<CardInfo>> cardInfos = phoneCardService.getListCard(request, bodyRequest);
        // decrypt data of serial and pin code
        if (!CollectionUtils.isEmpty(cardInfos.getData())) {
            for (CardInfo cardInfo : cardInfos.getData()) {
                cardInfo.setSerial(StringUtils.isEmpty(cardInfo.getSerial()) ? "" : cryptLib.decrypt(cardInfo.getSerial(), keyEncrytData, IvAESSecretKey));
                cardInfo.setPin_code(StringUtils.isEmpty(cardInfo.getPin_code()) ? "" : cryptLib.decrypt(cardInfo.getPin_code(), keyEncrytData, IvAESSecretKey));
            }
        }
        return new ResponseEntity<>(cardInfos, HttpStatus.OK);
    }

}
