package vn.com.seabank.biz.web.rest;

import com.codahale.metrics.annotation.Timed;
import com.seabank.biz.authen.bean.BodyConfirmOTP;
import com.seabank.biz.authen.bean.ConfirmOTPResponse;
import com.seabank.biz.cardopcde.bean.GetAcqPurchaseOTPReq;
import com.seabank.biz.cardopcde.bean.GetAcqPurchaseOTPRes;
import com.seabank.biz.cardopcde.bean.GetPartnerAcqPurchaseReq;
import com.seabank.biz.cardopcde.bean.GetPartnerAcqPurchaseRes;
import com.seabank.biz.custinfo.bean.UserProfileResponse;
import com.seabank.biz.utility.bean.ProviderServiceInfo;
import com.swib.seanet.t24enquiry.bean.PartnerServiceResponse;
import com.swib.seanet.t24enquiry.bean.SaleAccountInfo;
import com.swib.seanet.t24enquiry.bean.T24Header;
import com.swib.seanet.t24enquiry.service.T24EnquiryApiClient;
import com.swib.seanet.topup.bean.TopUpBodyRequest;
import com.swib.seanet.topup.bean.TopUpResponse;
import com.swib.seanet.topup.bean.TopUpTransactionBodyRequest;
import com.swib.seanet.topup.bean.TopUpTransactionResponse;
import com.swib.services.config.CacheConfiguration;
import com.swib.services.domain.VoucherBatch;
import com.swib.services.service.CustomerObserver;
import com.swib.services.service.PrePaidAccService;
import com.swib.services.service.SwibCommonService;
import com.swib.services.service.VoucherBatchService;
import com.swib.services.service.dto.UseVoucherRequestDTO;
import com.swib.services.service.dto.VoucherUsedDTO;
import com.swib.services.utils.SwibUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import vn.com.seabank.biz.bean.*;
import vn.com.seabank.biz.domain.RechargeCard;
import vn.com.seabank.biz.domain.ServiceParam;
import vn.com.seabank.biz.domain.UserSessionInfo;
import vn.com.seabank.biz.dto.GetPartnerTopUpMobileReq;
import vn.com.seabank.biz.dto.ResponseObject;
import vn.com.seabank.biz.exception.CoreApiException;
import vn.com.seabank.biz.service.TopUpService;
import vn.com.seabank.biz.utils.BuildHeaderUtils;

import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import static vn.com.seabank.biz.config.Constants.*;

/**
 * Created by sonba.itsol
 * Email: sonba.itsol@seabank.com.vn
 * Date: 3/23/2020 / 03 / 2020
 * Time: 10:18 AM
 */

@RestController
@RequestMapping(value = "/api/p0603-topup")
public class TopUpResource extends BuildHeaderUtils {
    private final Logger log = LoggerFactory.getLogger(TopUpResource.class);
    @Inject
    private TopUpService topUpService;

    private static final String VND = "VND";
    private static final String TOPUP_MOBILE = "TOPUP_MOBILE";
    private static final String TOPUP_SERVICE = "TOPUP_SERVICE";
    private static final String CODE99 = "099";
    private static final String BANKAPI_NEXTGEN = "BANKAPI-NEXTGEN";

    @Autowired
    VoucherBatchService voucherBatchService;

    @Autowired
    PrePaidAccService prePaidAccService;

    @Inject
    private CustomerObserver customerObserver;

    @Inject
    private SwibUtils swibUtils;

    private static final String CODE02 = "02";
    private static final String BANKAPI_ADMIN_VOUCHER = "BANKAPI-ADMIN-VOUCHER";

    /**
     * getAccountInfo - Lay danh sach tai khoan thanh toan theo cau hinh
     *
     * @param request
     * @return
     */
    @RequestMapping(value = "/get-account-info", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<Object>> getAccountInfoTopUpMobile(HttpServletRequest request) throws Exception {
        UserProfileResponse userProfileResponse = getUserProfile(request);
        String key = userProfileResponse.getPackAge();
        ResponseObject<Object> transaction = topUpService.getListAccountForTopUp(request, key);
        return new ResponseEntity<>(transaction, HttpStatus.OK);
    }

    /**
     * Truy van danh sach nha cung cap cua nap the dien thoai
     *
     * @param request
     * @return Map
     */
    @RequestMapping(value = "/get-provider-topup-mobile", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<List<ProviderServiceInfo>>> getServiceProvider(HttpServletRequest request) throws IOException {
        ResponseObject<List<ProviderServiceInfo>> providerService = topUpService.getProviderService(request, SERVICE_TOPUP_ID);
        return new ResponseEntity<>(providerService, HttpStatus.OK);
    }

    /**
     * getUserProfile - Lay thong tin khach hang
     *
     * @param httpServletRequest
     * @return
     */
    @RequestMapping(value = "/get-user-profile", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<UserProfileResponse>> getUserProfiles(HttpServletRequest httpServletRequest) {
        UserProfileResponse userProfileResponse = getUserProfile(httpServletRequest);
        return new ResponseEntity<>(ResponseObject.success(userProfileResponse), HttpStatus.OK);
    }

    /**
     * Truy van danh sach nha cung cap cua nap the dien thoai
     *
     * @param request
     * @return Map
     */
    @RequestMapping(value = {"/get-partner-topup-mobile"}, method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<PartnerServiceResponse>> getPartnerTopUpMobile(HttpServletRequest request, @RequestBody GetPartnerTopUpMobileReq bodyRequest) {
        ResponseObject<PartnerServiceResponse> partnerServiceResponse = topUpService.getPartnerTopUpService(request, bodyRequest.getProviderId());
        return new ResponseEntity<>(partnerServiceResponse, HttpStatus.OK);
    }

    /**
     * getListAmount
     *
     * @return
     */
    @RequestMapping(value = "/get-list-amount-topup-mobile", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<List<RechargeCardDTO>>> getListAmount() {
        List<RechargeCard> rechargeCards = (List<RechargeCard>) CacheConfiguration.getHazelcastInstance().getMap(ADMIN).get(RECHARGE_CARD_CACHE);
        List<RechargeCardDTO> rechargeCardDTOS = new ArrayList<>();
        rechargeCards.forEach(rc -> {
            rechargeCardDTOS.add(new RechargeCardDTO(rc));
        });
        rechargeCardDTOS.sort(new Comparator<RechargeCardDTO>() {
            @Override
            public int compare(RechargeCardDTO o1, RechargeCardDTO o2) {
                return o1.getAmount().compareTo(o2.getAmount());
            }
        });
        return new ResponseEntity<>(ResponseObject.success(rechargeCardDTOS), HttpStatus.OK);
    }

    /**
     * getAcqPurchaseOTP
     *
     * @return
     */
    @RequestMapping(value = "/get-acq-purchase-otp", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<GetAcqPurchaseOTPRes>> getAcqPurchaseOTP(HttpServletRequest request, @RequestBody GetAcqPurchaseOTPReq bodyRequest) throws Exception {
        ResponseObject<GetAcqPurchaseOTPRes> getAcqPurchaseOTPRes = topUpService.getAcqPurchaseOTP(request, bodyRequest);
        return new ResponseEntity<>(getAcqPurchaseOTPRes, HttpStatus.OK);
    }

    /**
     * getAcqPurchaseOTP
     *
     * @return
     */
    @RequestMapping(value = "/get-partner-acq-purchase", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<GetPartnerAcqPurchaseRes>> getPartnerAcqPurchase(HttpServletRequest request, @RequestBody GetPartnerAcqPurchaseReq bodyRequest) throws Exception {
        ResponseObject<GetPartnerAcqPurchaseRes> getPartnerAcqPurchaseRes = topUpService.getPartnerAcqPurchase(request, bodyRequest);
        return new ResponseEntity<>(getPartnerAcqPurchaseRes, HttpStatus.OK);
    }

    /**
     * topupTransaction
     *
     * @return
     */
    @RequestMapping(value = "/topup-transaction", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<TopUpTransactionResponse>> topUpTransaction(HttpServletRequest request, @RequestBody TopUpTransactionBodyRequest bodyRequest) throws IOException {
        log.info("-----> START TOP UP:: ");
        VoucherBatch validVoucher = null;
        if (prePaidAccService.checkLimitRemainPrePaidCard(bodyRequest.getDebitAccount(), Long.valueOf(bodyRequest.getTransAmount()))) {
            //check voucher information
            if (!StringUtils.isEmpty(bodyRequest.getRegIssuanceName())) {
                validVoucher = voucherBatchService.getValidVoucher(request, bodyRequest.getRegIssuanceName(), TOPUP_MOBILE);
                if (validVoucher == null) {
                    throw new CoreApiException(CODE02, "EVoucherCode is not exist or wrong type", BANKAPI_ADMIN_VOUCHER);
                }
            }
            //Verify OTP
            confirmOTP(request, bodyRequest.getOtp());

            //Mock Response
            ResponseObject<TopUpTransactionResponse> mock = mockResponseTopUpGW(bodyRequest.getTelNo());
            if (mock != null) {
                return new ResponseEntity<>(mock, HttpStatus.OK);
            }

            // check topup by credit card or account
            if (StringUtils.isEmpty(bodyRequest.getTokenCard())) {
                // Kiem tra tai khoan lay sao ke phai nam
                // trong danh sach tai khoan cua nguoi dung
                checkAccountValid(request, bodyRequest.getDebitAccount());
            } else {
                // get sale account when topup by credit card
                T24Header header = createT24Header(request);
                T24EnquiryApiClient t24EnquiryApiClient = new T24EnquiryApiClient(header);
                SaleAccountInfo saleAccountInfo = t24EnquiryApiClient.getSaleAccByTokenCard(bodyRequest.getTokenCard());
                bodyRequest.setDebitAccount(saleAccountInfo.getAccountId());
            }
            ResponseObject<TopUpTransactionResponse> transaction = topUpService.topUpTransaction(request, bodyRequest);
            customerObserver.onAccountChanged(swibUtils.getUserSessionInfo(request));
            try {
                if (validVoucher != null) {
                    UseVoucherRequestDTO useVoucherRequestDTO = new UseVoucherRequestDTO();
                    useVoucherRequestDTO.setAccountNo(bodyRequest.getDebitAccount());
                    useVoucherRequestDTO.setCreditCcy(VND);
                    useVoucherRequestDTO.setDebitCcy(VND);
                    useVoucherRequestDTO.setEvoucherCode(validVoucher.getEvoucherCode());
                    useVoucherRequestDTO.setTransId(transaction.getData().getTransactionNO());
                    useVoucherRequestDTO.setTransAmt(bodyRequest.getTransAmount());
                    useVoucherRequestDTO.setTransTypeReq(TOPUP_MOBILE);
                    voucherBatchService.usedVoucher(request, useVoucherRequestDTO);
                }
            } catch (Exception ex) {
                log.error("-----> TOP UP ERROR:: ", ex.getMessage());
            }
            transaction.getData().setCurrency(VND);
            log.info("-----> TOP UP SUCCESS :: " + transaction.getData().getTransactionNO());
            return new ResponseEntity<>(transaction, HttpStatus.OK);
        } else {
            throw new CoreApiException(CODE99, "Prepaid card remain amount not enough", BANKAPI_NEXTGEN);
        }
    }

    /**
     * topup Transaction by VNPT
     *
     * @return
     */
    @RequestMapping(value = "/vnpt-topup-transaction", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<TopUpResponse>> topUpTransactionByVNPT(HttpServletRequest request, @RequestBody TopUpBodyRequest bodyRequest) throws IOException {
        log.info("-----> START VNPT TOP UP:: ");
        VoucherBatch validVoucher = null;
        if (prePaidAccService.checkLimitRemainPrePaidCard(bodyRequest.getDebitAccount(), Long.valueOf(bodyRequest.getBillAmount()))) {

            //check voucher information
            if (!StringUtils.isEmpty(bodyRequest.getRegIssuanceName())) {
                validVoucher = voucherBatchService.getValidVoucher(request, bodyRequest.getRegIssuanceName(), TOPUP_MOBILE);
                if (validVoucher == null) {
                    throw new CoreApiException(CODE02, "EVoucherCode is not exist or wrong type", BANKAPI_ADMIN_VOUCHER);
                }
            }
            //Verify OTP
            confirmOTP(request, bodyRequest.getOtp());

            //Mock Response
            ResponseObject<TopUpResponse> mock = mockResponseTopUpVNPT(bodyRequest.getPaymentCode());
            if (mock != null) {
                return new ResponseEntity<>(mock, HttpStatus.OK);
            }

            // check topup by credit card or account
            if (StringUtils.isEmpty(bodyRequest.getTokenCard())) {
                // Kiem tra tai khoan lay sao ke phai nam
                // trong danh sach tai khoan cua nguoi dung
                checkAccountValid(request, bodyRequest.getDebitAccount());
            } else {
                // get sale account when topup by credit card
                T24Header header = createT24Header(request);
                T24EnquiryApiClient t24EnquiryApiClient = new T24EnquiryApiClient(header);
                SaleAccountInfo saleAccountInfo = t24EnquiryApiClient.getSaleAccByTokenCard(bodyRequest.getTokenCard());
                bodyRequest.setDebitAccount(saleAccountInfo.getAccountId());
            }
            ResponseObject<TopUpResponse> transaction = topUpService.topUpTransactionByVNPT(request, bodyRequest);
            customerObserver.onAccountChanged(swibUtils.getUserSessionInfo(request));
            try {
                if (validVoucher != null) {
                    UseVoucherRequestDTO useVoucherRequestDTO = new UseVoucherRequestDTO();
                    useVoucherRequestDTO.setAccountNo(bodyRequest.getDebitAccount());
                    useVoucherRequestDTO.setCreditCcy(VND);
                    useVoucherRequestDTO.setDebitCcy(VND);
                    useVoucherRequestDTO.setEvoucherCode(validVoucher.getEvoucherCode());
                    useVoucherRequestDTO.setTransId(transaction.getData().getTrans_response_id());
                    useVoucherRequestDTO.setTransAmt(bodyRequest.getBillAmount());
                    useVoucherRequestDTO.setTransTypeReq(TOPUP_MOBILE);
                    voucherBatchService.usedVoucher(request, useVoucherRequestDTO);
                }
            } catch (Exception ex) {
                log.error("-----> VNPT TOP UP ERROR:: ", ex.getMessage());
            }
            transaction.getData().setCurrency(VND);
            log.info("-----> VNPT TOP UP SUCCESS:: " + transaction.getData().getTrans_response_id());
            return new ResponseEntity<>(transaction, HttpStatus.OK);
        } else {
            throw new CoreApiException(CODE99, "Prepaid card remain amount not enough", BANKAPI_NEXTGEN);
        }
    }

    /**
     * getServiceChildren - Truy van danh sach dich vu con cua dich vu BILLING
     *
     * @param request
     */
    @RequestMapping(value = "/get-list-topup-service-provider", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<List<ServiceParam>>> getListTopUpServiceProvider(HttpServletRequest request) throws IOException {
        ResponseObject<List<ServiceParam>> services = topUpService.getListTopUpService(request);
        return new ResponseEntity<>(services, HttpStatus.OK);
    }

    @RequestMapping(value = "/get-vetc-amount-fee", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<VETCGetAmountResponse>> getVETCAmount(HttpServletRequest request, @RequestBody VETCGetAmountParam vetcGetAmountParam) throws Exception {
        vetcGetAmountParam.setUserId(getUsername(request));
        vetcGetAmountParam.setProviderId("VETC");
        ResponseObject<VETCGetAmountResponse> vetcGetAmountResponse = topUpService.getVETCAmountFee(request, vetcGetAmountParam);
        return new ResponseEntity<>(vetcGetAmountResponse, HttpStatus.OK);
    }

    @RequestMapping(value = "/transaction-vetc-amount", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<VETCTransactionResponse>> transactionVETCAmount(HttpServletRequest request, @RequestBody VETCTransactionParam vetcParams) throws Exception {
        log.info("-----> START VETC:: ");
        VoucherBatch validVoucher = null;
        if (prePaidAccService.checkLimitRemainPrePaidCard(vetcParams.getCustomerAccount(), Long.valueOf(vetcParams.getAmount()))) {
            if (!StringUtils.isEmpty(vetcParams.getRegIssuanceName())) {
                validVoucher = voucherBatchService.getValidVoucher(request, vetcParams.getRegIssuanceName(), TOPUP_SERVICE);
                if (validVoucher == null) {
                    throw new CoreApiException(CODE02, "EVoucherCode is not exist or wrong type", BANKAPI_ADMIN_VOUCHER);
                }
            }
            //Verify OTP
            confirmOTP(request, vetcParams.getOtp());

            vetcParams.setUserId(getUsername(request));
            vetcParams.setPassword(getPassword(request));
            vetcParams.setProviderId("VETC");
            ResponseObject<VETCTransactionResponse> vetcGetAmountResponse = topUpService.transactionVETCAmount(request, vetcParams);
            customerObserver.onAccountChanged(swibUtils.getUserSessionInfo(request));
            try {
                if (validVoucher != null) {
                    UseVoucherRequestDTO useVoucherRequestDTO = new UseVoucherRequestDTO();
                    useVoucherRequestDTO.setAccountNo(vetcParams.getCustomerAccount());
                    useVoucherRequestDTO.setCreditCcy(VND);
                    useVoucherRequestDTO.setDebitCcy(VND);
                    useVoucherRequestDTO.setEvoucherCode(validVoucher.getEvoucherCode());
                    useVoucherRequestDTO.setTransId(vetcGetAmountResponse.getData().getFtTransaction());
                    useVoucherRequestDTO.setTransAmt(vetcParams.getAmount());
                    useVoucherRequestDTO.setTransTypeReq(TOPUP_SERVICE);
                    voucherBatchService.usedVoucher(request, useVoucherRequestDTO);
                }
            } catch (Exception ex) {
                log.error("-----> VETC ERROR:: ", ex.getMessage());
            }

            vetcGetAmountResponse.getData().setCurrency(VND);
            log.info("-----> VETC SUCCESS:: " + vetcGetAmountResponse.getData().getFtTransaction());
            return new ResponseEntity<>(vetcGetAmountResponse, HttpStatus.OK);
        } else {
            throw new CoreApiException(CODE99, "Prepaid card remain amount not enough", BANKAPI_NEXTGEN);
        }
    }
}
