package vn.com.seabank.biz.service;

import com.seabank.biz.custinfo.bean.AccountInfo;
import com.seabank.biz.sto.bean.*;
import com.seabank.biz.sto.service.StoService;
import com.seabank.biz.utility.bean.*;
import com.seabank.biz.utility.service.UtilityAPIClient;
import com.swib.seanet.topup.bean.BillingEnquiryBodyRequest;
import com.swib.seanet.topup.bean.BillingTransactionBodyRequest;
import com.swib.seanet.topup.bean.BillingTransactionResponse;
import com.swib.seanet.topup.bean.DocumentInfo;
import com.swib.seanet.topup.dto.VNPTHeaderReq;
import com.swib.seanet.topup.dto.request.GetVNPTBillingInfoReq;
import com.swib.seanet.topup.dto.request.VNPTBillingTransactionReq;
import com.swib.seanet.topup.dto.response.GetVNPTBillingInfoRes;
import com.swib.seanet.topup.dto.response.VNPTBillingTransactionRes;
import com.swib.seanet.topup.dto.response.VNPTProvincesRes;
import com.swib.seanet.topup.service.TopUpApiClient;
import com.swib.seanet.topup.service.VNPTApiClient;
import com.swib.services.config.CacheConfiguration;
import com.swib.services.service.SWIBEnquiryService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import vn.com.seabank.biz.domain.*;
import vn.com.seabank.biz.dto.BillingEnquiryReq;
import vn.com.seabank.biz.dto.BillingTransactionReq;
import vn.com.seabank.biz.dto.ResponseObject;
import vn.com.seabank.biz.dto.UserLoginDTO;
import vn.com.seabank.biz.utils.BuildHeaderUtils;

import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.TimeUnit;

import static vn.com.seabank.biz.config.Constants.*;

/**
 * Created by sonba.itsol
 * Email: sonba.itsol@seabank.com.vn
 * Date: 3/24/2020 / 03 / 2020
 * Time: 4:12 PM
 */
@Service
public class BillingService extends BuildHeaderUtils {

    private final Logger LOGGER = LoggerFactory.getLogger(BillingService.class);

    @Inject
    private SWIBEnquiryService swibEnquiryService;

    /**
     * Get list account for topup function
     */
    public ResponseObject<Object> getListAccountForBilling(HttpServletRequest request, String packAge) throws Exception {
        List<AccountInfo> result = new ArrayList<>();
        Map<String, Object> response = new HashMap<>();
        List<AccountInfo> lstAccount = swibEnquiryService.listCustomerAccounts(getUserSessionInfo(request), true);
        List<FtCfgCache> ftCfgCaches = (List<FtCfgCache>) CacheConfiguration.getHazelcastInstance().getMap("ADMIN").get("TRANSFER_CACHE");
        String ftTypeId;
        List<FtRule> ftRules = new ArrayList<>();
        for (FtCfgCache ftCfgCache : ftCfgCaches) {
            FtCfg ftCfg = ftCfgCache.getFtCfg();
            ftTypeId = ftCfg.getFtTypeId().getFtTypeId();
            String spId = ftCfg.getSpId().getSpId();
            if (ftTypeId.equals(PAYMENT_BILLING) && spId.equals(packAge) && ftCfgCache.getFtRule() != null) {
                ftRules.addAll(ftCfgCache.getFtRule());
                FtCfg regexCfg = ftCfgCache.getFtCfg();
                regexCfg.getFtTypeId().setFtCfgs(null);
                regexCfg.getSpId().setFtCfgs(null);
                response.put("regexCfg", regexCfg);
            }
        }

        for (FtRule ftRule : ftRules) {
            AccGrp accGrp = ftRule.getFromAccGrp();
            StringBuilder tempAccExclusive = new StringBuilder();
            if (accGrp.getAccountExclusive() != null && !accGrp.getAccountExclusive().isEmpty()) {
                tempAccExclusive.append(accGrp.getAccountExclusive()).append(",");
            }
            String[] lstAccCat = (accGrp.getAccCat() + ",").split("\\,");
            List<String> lstCatDisplayAsChart = Arrays.asList(lstAccCat);

            String[] lstAccCurrency = (accGrp.getCurency() + ",").split("\\,");
            List<String> currency = Arrays.asList(lstAccCurrency);

            String[] lstAccExclusive = tempAccExclusive.toString().split("\\,");
            List<String> accExclusives = Arrays.asList(lstAccExclusive);

            result.addAll(checkAccList(lstAccount, lstCatDisplayAsChart, currency, accExclusives));
        }
        result = new ArrayList<>(new LinkedHashSet<>(result));
        response.put("accountList", result);
        return ResponseObject.success(response);
    }

    /**
     * checkAccList
     */
    private List<AccountInfo> checkAccList(List<AccountInfo> lstAccount, List<String> lstCatDisplayAsChart, List<String> curency, List<String> accExclusives) {
        List<AccountInfo> result = new ArrayList<>();
        for (AccountInfo accountInfo : lstAccount) {
            if (trollAcc(lstCatDisplayAsChart, accountInfo) && trollCurrency(curency, accountInfo) && trollMethod(accExclusives, accountInfo)) {
                result.add(accountInfo);
            }
        }
        return result;
    }

    /**
     * Phuong thuc duoc tach ra de check getCategory
     */
    private Boolean trollAcc(List<String> lstCatDisplayAsCharts, AccountInfo accountInfo) {
        for (String lstCatDisplayAsChart : lstCatDisplayAsCharts) {
            if (!lstCatDisplayAsChart.equals("") && lstCatDisplayAsChart.trim().equals(accountInfo.getProductCode().trim())) {
                return true;
            }
        }
        return false;
    }

    /**
     * Phuong thuc duoc tach ra de check currencys
     */
    private Boolean trollCurrency(List<String> currencys, AccountInfo accountInfo) {
        for (String currency : currencys) {
            if (!currency.equals("") && currency.trim().equals(accountInfo.getCurrency().trim())) {
                return true;
            }
        }
        return false;
    }

    /**
     * Phuong thuc duoc tach ra de check tai khoan loai tru
     */
    private Boolean trollMethod(List<String> accExclusives, AccountInfo accountInfo) {
        for (String accExclusive : accExclusives) {
            if (!accExclusive.equals("") && accountInfo.getAccountID().indexOf(accExclusive) == 0) {
                return false;
            }
        }
        return true;
    }

    /**
     * billingEnquiry - Truy van thong tin hoa don
     */
    public ResponseObject<DocumentInfo> billingEnquiry(HttpServletRequest request, BillingEnquiryReq billingEnquiryReq) {
        BillingEnquiryBodyRequest bodyRequest = billingEnquiryReq.copyTo(getUserLoginDTO(request));
        TopUpApiClient topUpApiClient = new TopUpApiClient();
        return ResponseObject.success(topUpApiClient.BillingEnquiry(bodyRequest).getDocumentInfo());
    }

    /**
     * billingTransaction - Thanh toan hoa don
     */
    public ResponseObject<BillingTransactionResponse> billingTransaction(HttpServletRequest request, BillingTransactionReq billingTransactionReq) {
        UserLoginDTO userLoginDTO = getUserLoginDTO(request);
        BillingTransactionBodyRequest bodyRequest = billingTransactionReq.copyTo(userLoginDTO);
        TopUpApiClient topUpApiClient = new TopUpApiClient();
        BillingTransactionResponse billingTransactionResponse = topUpApiClient.BillingTransaction(bodyRequest);
        // check discount remain limit credit card when transfer successful
//        if (!StringUtils.isEmpty(billingTransactionReq.getTokenCard())) {
//            updateLimitRemainCredit(billingTransactionReq.getTokenCard(), userLoginDTO.getUserName(), billingTransactionReq.getAmount());
//        }
        return ResponseObject.success(billingTransactionResponse);
    }

    /**
     * getVNPTBillingInfo
     */
    public ResponseObject<GetVNPTBillingInfoRes> getVNPTBillingInfo(HttpServletRequest request, GetVNPTBillingInfoReq getVNPTBillingInfoRes) {
        VNPTHeaderReq vnptHeaderReq = createdBillingVNPTHeaderReq(request, null);
        VNPTApiClient vnptApiClient = new VNPTApiClient();
        return vnptApiClient.getVNPTBillingInfo(vnptHeaderReq, getVNPTBillingInfoRes);
    }

    /**
     * vnptBillingTransaction
     */
    public ResponseObject<VNPTBillingTransactionRes> vnptBillingTransaction(HttpServletRequest request, VNPTBillingTransactionReq vnptBillingTransactionReq) {
        String userName = getUsername(request);
        vnptBillingTransactionReq.setSessionId(getSessionId(request));
        vnptBillingTransactionReq.setUserName(userName);
        VNPTHeaderReq vnptHeaderReq = createdBillingVNPTHeaderReq(request, "RELIEF_AUTH_SN2");
        VNPTApiClient vnptApiClient = new VNPTApiClient();
        ResponseObject<VNPTBillingTransactionRes> res = vnptApiClient.VNPTBillingTransaction(vnptHeaderReq, vnptBillingTransactionReq);

        // check discount remain limit credit card when transfer successful
//        if (!StringUtils.isEmpty(vnptBillingTransactionReq.getTokenCard())) {
//            updateLimitRemainCredit(vnptBillingTransactionReq.getTokenCard(), userName, vnptBillingTransactionReq.getBillAmount());
//        }
        return res;
    }

    /**
     * discount remain limit credit card when transfer successful
     */
//    private void updateLimitRemainCredit(String tokenCard, String username, String amount) {
//        try {
//            List<Card> lstCardEfficiency = (List<Card>) CacheConfiguration.getHazelcastInstance().getMap(LIST_CARD_FULL).get(CUSTOMER_ + username);
//            LOGGER.info("lstCardEfficiency NULL: {}", lstCardEfficiency == null);
//            for (int i = 0; i < Objects.requireNonNull(lstCardEfficiency).size(); i++) {
//                if (tokenCard.equals(lstCardEfficiency.get(i).getTokenCard())) {
//                    // discount remain limit credit card when transfer successful
//                    Double limitRemain = Double.parseDouble(lstCardEfficiency.get(i).getLimitRemain()) - Double.parseDouble(amount);
//                    lstCardEfficiency.get(i).setLimitRemain(String.valueOf(limitRemain));
//                    CacheConfiguration.getHazelcastInstance().getMap(LIST_CARD_FULL).put(CUSTOMER_ + username, lstCardEfficiency, 30, TimeUnit.MINUTES);
//                    break;
//                }
//            }
//        } catch (Exception e) {
//            LOGGER.error("updateLimitRemainCredit: ", e);
//        }
//    }

    /**
     * discount remain limit credit card when transfer successful
     */
//    private void revertLimitRemainCredit(String tokenCard, String username, String amount) {
//        try {
//            List<Card> lstCardEfficiency = (List<Card>) CacheConfiguration.getHazelcastInstance().getMap(LIST_CARD_FULL).get(CUSTOMER_ + username);
//            for (int i = 0; i < lstCardEfficiency.size(); i++) {
//                if (tokenCard.equals(lstCardEfficiency.get(i).getTokenCard())) {
//                    // discount remain limit credit card when transfer successful
//                    Double limitRemain = Double.parseDouble(lstCardEfficiency.get(i).getLimitRemain()) + Double.parseDouble(amount);
//                    lstCardEfficiency.get(i).setLimitRemain(String.valueOf(limitRemain));
//                    CacheConfiguration.getHazelcastInstance().getMap(LIST_CARD_FULL).put(CUSTOMER_ + username, lstCardEfficiency, 30, TimeUnit.MINUTES);
//                    break;
//                }
//            }
//        } catch (NumberFormatException e) {
//            //Log
//        }
//    }

    /**
     * getVNPTBillingCityList
     */
    public ResponseObject<List<ProvinceVNPT>> getVNPTBillingCityList(HttpServletRequest request) {
        List<ProvinceVNPT> provinceVNPTS = null;
        try {
            provinceVNPTS = (List<ProvinceVNPT>) CacheConfiguration.getHazelcastInstance().getMap("ADMIN").get("TOPUP_CFG_CACHE_PROVINCES_VNPT");
        } catch (Exception e) {
            CacheConfiguration.getHazelcastInstance().getMap("ADMIN").remove("TOPUP_CFG_CACHE_PROVINCES_VNPT");
        }
        if (provinceVNPTS == null || provinceVNPTS.isEmpty()) {
            VNPTHeaderReq headerReq = createdBillingVNPTHeaderReq(request, null);
            VNPTApiClient vnptApiClient = new VNPTApiClient();
            ResponseObject<List<VNPTProvincesRes.Province>> res = vnptApiClient.getVNPTProvinces(headerReq);
            provinceVNPTS = new ArrayList<>();
            for (VNPTProvincesRes.Province province : res.getData()) {
                provinceVNPTS.add(new ProvinceVNPT(province.getPostalCode(), province.getProvinceId(), province.getProvinceName()));
            }
            CacheConfiguration.getHazelcastInstance().getMap("ADMIN").remove("TOPUP_CFG_CACHE_PROVINCES_VNPT");
            CacheConfiguration.getHazelcastInstance().getMap("ADMIN").put("TOPUP_CFG_CACHE_PROVINCES_VNPT", provinceVNPTS);
        }
        return ResponseObject.success(provinceVNPTS);
    }

    /**
     * getServiceProvider - Truy van danh sach dich vu va nha cung cap dich vu
     */
    public ResponseObject<Object> getServiceProvider(HttpServletRequest request) throws IOException {
        Map<String, Object> result = new HashMap<>();
        List<ServiceParam> transaction = new ArrayList<>();
        List<ServiceInfo> serviceInfos;
        List<Topup> topups = null;
        try {
            topups = (List<Topup>) CacheConfiguration.getHazelcastInstance().getMap("ADMIN").get("TOPUP_CFG_CACHE");
        } catch (Exception e) {
            CacheConfiguration.getHazelcastInstance().getMap("ADMIN").remove("TOPUP_CFG_CACHE");
            topups = new ArrayList<>();
        }
        if (!CollectionUtils.isEmpty(topups)) {
            for (Topup topup : topups) {
                ServiceParam temp = new ServiceParam(topup);
                transaction.add(temp);
            }
        } else {
            serviceInfos = getService(request).getData();
            for (ServiceInfo serviceInfo : serviceInfos) {
                ProviderServiceBodyRequest providerServiceBodyRequest = new ProviderServiceBodyRequest();
                providerServiceBodyRequest.setServiceID(serviceInfo.getServiceID());
                List<ProviderServiceInfo> providerServiceInfos = getProviderService(request, providerServiceBodyRequest).getData();
                if (providerServiceInfos != null) {
                    for (ProviderServiceInfo providerServiceInfo : providerServiceInfos) {
                        ServiceParam temp = new ServiceParam(serviceInfo, providerServiceInfo);
                        transaction.add(temp);
                    }
                }
            }
        }
        result.put("serviceProvider", transaction);
        List<Object[]> lstService = (List<Object[]>) CacheConfiguration.getHazelcastInstance().getMap("ADMIN").get("TOPUP_CFG_CACHE_LSTSERVICE");
        if (CollectionUtils.isEmpty(lstService)) {
            serviceInfos = getService(request).getData();
            Object[] temp = new Object[serviceInfos.size()];
            int i = 0;
            for (ServiceInfo serviceInfo : serviceInfos) {
                temp[i] = serviceInfo.getServiceID();
                i++;
            }
            lstService.add(temp);
            result.put("lstService", lstService);
            CacheConfiguration.getHazelcastInstance().getMap("ADMIN").put("TOPUP_CFG_CACHE_LSTSERVICE", lstService);
        } else {
            result.put("lstService", lstService);
        }
        return ResponseObject.success(result);
    }

    /**
     * getService - Truy van danh sach cac dich vu con cua dich vu BILLING
     */
    private ResponseObject<List<ServiceInfo>> getService(HttpServletRequest request) throws IOException {
        List<ServiceInfo> transaction = null;
        try {
            transaction = CacheConfiguration.getHazelcastInstance().getList("UTILITY_LIST_SERVICE");
        } catch (Exception e) {
            CacheConfiguration.getHazelcastInstance().getList("UTILITY_LIST_SERVICE").clear();
        }
        if (CollectionUtils.isEmpty(transaction)) {
            UtilityHeaderRequest headerRequest = createUtilityHeader(request);
            ServiceBodyRequest bodyRequest = new ServiceBodyRequest();
            bodyRequest.setAuthenType(GET_SERVICES);
            bodyRequest.setEpayType(BILLING);
            UtilityAPIClient utilityAPIClient = new UtilityAPIClient();
            ServiceResponse serviceResponse = utilityAPIClient.getSERVICES(headerRequest, bodyRequest);
            if (serviceResponse != null && serviceResponse.getResponseCode() != null && serviceResponse.getResponseCode().equals("00")) {
                transaction = serviceResponse.getService();
                CacheConfiguration.getHazelcastInstance().getList("UTILITY_LIST_SERVICE").add(transaction);
            }
        }
        return ResponseObject.success(transaction);
    }

    /**
     * getProviderService - Truy van danh sach cac nha cung cap cua mot dich vu
     */
    private ResponseObject<List<ProviderServiceInfo>> getProviderService(HttpServletRequest request, ProviderServiceBodyRequest bodyRequest) throws IOException {
        List<ProviderServiceInfo> transaction = null;
        try {
            transaction = CacheConfiguration.getHazelcastInstance().getList("UTILITY_LIST_PROVIDER." + bodyRequest.getServiceID());
        } catch (Exception e) {
            CacheConfiguration.getHazelcastInstance().getList("UTILITY_LIST_PROVIDER." + bodyRequest.getServiceID()).clear();
        }
        if (CollectionUtils.isEmpty(transaction)) {
            UtilityHeaderRequest headerRequest = createUtilityHeader(request);
            bodyRequest.setAuthenType(GET_PROVIDER_SERVICE);
            UtilityAPIClient utilityAPIClient = new UtilityAPIClient();
            ProviderServiceResponse providerServiceResponse = utilityAPIClient.getProviderService(headerRequest, bodyRequest);
            if (providerServiceResponse != null && providerServiceResponse.getResponseCode() != null && providerServiceResponse.getResponseCode().equals("00")) {
                transaction = providerServiceResponse.getProvider();
                CacheConfiguration.getHazelcastInstance().getList("UTILITY_LIST_PROVIDER." + bodyRequest.getServiceID()).addAll(transaction);
            }
        }
        return ResponseObject.success(transaction);
    }

    /**
     * getListSTO
     */
    public ResponseObject<Object> getListSTO(HttpServletRequest request) throws Exception {
        List stoLst = new ArrayList<>();
        String key = STO_LST + getUsername(request);
        Object objectCache = CacheConfiguration.getHazelcastInstance().getMap(STO_ENQUIRY).get(key);
        if (objectCache != null) {
            stoLst = (List) objectCache;
        }
        if (stoLst.size() < 1) {
            StoHeaderRequest headerRequest = createSTOHeader(request);
            STO4BodyRequest bodyRequest = new STO4BodyRequest();
            bodyRequest.setSessionId(getSessionId(request));
            bodyRequest.setCustomerID(getCustomerId(request));
            bodyRequest.setStoID("");
            bodyRequest.setCreditAccount("");
            bodyRequest.setDebitAccount("");
            StoService stoService = new StoService();
            stoLst = stoService.STO4(headerRequest, bodyRequest);
            if (stoLst != null && stoLst.size() > 0) {
                CacheConfiguration.getHazelcastInstance().getMap(STO_ENQUIRY).put(key, stoLst, TIME_CACHE_STO, TimeUnit.DAYS);
            }
        }
        return ResponseObject.success(stoLst);
    }

    /**
     * getTransBilling - Tao ban ghi khong lan duyet
     */
    public ResponseObject<TransBillingResponse> getTransBilling(HttpServletRequest request, TransBillingBodyRequest bodyRequest, boolean isVNPTBilling) throws Exception {
        TransBillingResponse transaction;
        StoHeaderRequest headerRequest = createSTOHeader(request);
        if (bodyRequest.getEndDate() == null) {
            bodyRequest.setEndDate("");
        }
        if (bodyRequest.getDescription() == null) {
            bodyRequest.setDescription("");
        }
        if (bodyRequest.getTransDate() == null || bodyRequest.getTransDate().equalsIgnoreCase("")) {
            bodyRequest.setTransDate("");
        } else {
            if (bodyRequest.getTransDate().length() < 2) {
                bodyRequest.setTransDate("0" + bodyRequest.getTransDate());
            }
        }
        StoService stoService = new StoService();
        if (isVNPTBilling) {
            transaction = stoService.getTransVNPTBilling(headerRequest, bodyRequest);
        } else {
            transaction = stoService.getTransBilling(headerRequest, bodyRequest);
        }
        if (transaction.getResponseCode() != null && transaction.getResponseCode().equals("00")) {
            String key = STO_LST + getUsername(request);
            CacheConfiguration.getHazelcastInstance().getMap(STO_ENQUIRY).removeAsync(key);
        }
        return ResponseObject.success(transaction);
    }

    /**
     * getBillingChange - Chinh sua ban ghi (1 lan duyet - input)
     */
    public ResponseObject<TransBillingChangeResponse> getBillingChange(HttpServletRequest request, TransBillingChangeBodyRequest bodyRequest, boolean isVNPTBilling) throws Exception {
        TransBillingChangeResponse transaction;
        StoHeaderRequest headerRequest = createSTOHeader(request);
        if (bodyRequest.getEndDate() == null || bodyRequest.getEndDate().equals("")) {
            bodyRequest.setEndDate("");
        }
        if (bodyRequest.getDescription() == null) {
            bodyRequest.setDescription("");
        }
        if (bodyRequest.getTransDate() == null) {
            bodyRequest.setTransDate("");
        }
        bodyRequest.setServiceID(bodyRequest.getServiceID() == null ? "" : bodyRequest.getServiceID());
        bodyRequest.setProviderID(bodyRequest.getProviderID() == null ? "" : bodyRequest.getProviderID());
        StoService stoService = new StoService();
        if (isVNPTBilling) {
            transaction = stoService.getTransVNPTBillingChange(headerRequest, bodyRequest);
        } else {
            transaction = stoService.getTransBillingChange(headerRequest, bodyRequest);
        }
        if (transaction.getResponseCode() != null && transaction.getResponseCode().equals("00")) {
            String key = STO_LST + getUsername(request);
            CacheConfiguration.getHazelcastInstance().getMap(STO_ENQUIRY).removeAsync(key);
        }
        return ResponseObject.success(transaction);
    }

}
