package vn.com.seabank.biz.utils;

import com.seabank.biz.authen.bean.AccountHeader;
import com.seabank.biz.authen.bean.BodyConfirmOTP;
import com.seabank.biz.authen.bean.ConfirmOTPResponse;
import com.seabank.biz.custinfo.bean.UserProfileResponse;
import com.seabank.biz.sto.bean.StoHeaderRequest;
import com.seabank.biz.utility.bean.UtilityHeaderRequest;
import com.swib.core.sso.casclient.userdetails.CasToken;
import com.swib.seanet.ben.bean.BenHeaderRequest;
import com.swib.seanet.t24enquiry.bean.T24Header;
import com.swib.seanet.topup.bean.QRHeaderRequest;
import com.swib.seanet.topup.dto.VNPTHeaderReq;
import com.swib.services.config.ApiClientProperties;
import com.swib.services.config.CacheConfiguration;
import com.swib.services.service.SWIBEnquiryService;
import com.swib.services.service.SwibCommonService;
import com.swib.services.utils.SwibUtils;
import com.swib.services.validators.CurrentAccountValidator;
import com.swib.services.validators.PersonalValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import vn.com.seabank.biz.config.ConfigProperties;
import vn.com.seabank.biz.domain.UserSessionInfo;
import vn.com.seabank.biz.dto.UserLoginDTO;
import vn.com.seabank.biz.exception.CoreApiException;
import vn.com.seabank.biz.exception.SwibApiException;

import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by sonba.itsol
 * Email: sonba.itsol@seabank.com.vn
 * Date: 3/23/2020 / 03 / 2020
 * Time: 9:37 AM
 */

@Component
public class BuildHeaderUtils {

    @Inject
    private SwibUtils swibUtils;

    @Inject
    private ConfigProperties configProperties;

    @Inject
    private ApiClientProperties apiClientProperties;

    @Inject
    private SWIBEnquiryService swibEnquiryService;

    @Inject
    private SwibCommonService swibCommonService;

    @Autowired
    private CurrentAccountValidator currentAccountValidator;

    private static final String REQUEST_HEADER_ATTRIBUTE_API = "api";
    private static final String REQUEST_HEADER_ATTRIBUTE_API_KEY = "apiKey";
    private static final String REQUEST_HEADER_ATTRIBUTE_COMMAND = "command";
    private static final String REQUEST_HEADER_ATTRIBUTE_REQ_TIME = "req_time";
    private static final String REQUEST_HEADER_ATTRIBUTE_SENDER = "sender";
    private static final String REQUEST_HEADER_ATTRIBUTE_TRACEID = "traceid";
    private static final String REQUEST_HEADER_ATTRIBUTE_USERID = "userID";

    private static final String REQUEST_BODY_ATTRIBUTE_CHANNEL = "CHANNEL";
    private static final String REQUEST_BODY_ATTRIBUTE_PROVINCE_ID = "PROVINCE_ID";
    private static final String REQUEST_BODY_ATTRIBUTE_BILL_MONTH = "BILL_MONTH";
    private static final String REQUEST_BODY_ATTRIBUTE_CUSTOMER_ID = "CUSTOMER_ID";
    private static final String REQUEST_BODY_ATTRIBUTE_REGION_ID = "REGION_ID";
    private static final String REQUEST_BODY_ATTRIBUTE_ACCOUNT = "ACCOUNT";
    private static final String REQUEST_BODY_ATTRIBUTE_BILL_AMOUNT = "BILL_AMOUNT";
    private static final String REQUEST_BODY_ATTRIBUTE_SETTLEMENT_AMOUNT = "SETTLEMENT_AMOUNT";
    private static final String REQUEST_BODY_ATTRIBUTE_TRANS_REQUEST_ID = "TRANS_REQUEST_ID";
    private static final String REQUEST_BODY_ATTRIBUTE_TRANS_ADDITIONAL_INFO = "ADDITIONAL_INFO";
    private static final String REQUEST_BODY_ATTRIBUTE_TRANS_OPTIONS = "OPTIONS";
    private static final String REQUEST_BODY_ATTRIBUTE_USERSEANET = "USERSEANET";
    private static final String REQUEST_BODY_ATTRIBUTE_SESSION_ID = "SESSION_ID";
    private static final String REQUEST_BODY_ATTRIBUTE_COMPANY = "COMPANY";

    /**
     * getUserSessionInfo
     */
    public UserSessionInfo getUserSessionInfo(HttpServletRequest request) {
        return swibUtils.getUserSessionInfo(request);
    }

    /**
     * getSessionId
     */
    public String getSessionId(HttpServletRequest request) {
        return swibUtils.getSessionId(request);
    }

    /**
     * getTransId
     */
    public String getTransId() {
        return java.util.UUID.randomUUID().toString().replace("-", "").toUpperCase();
    }

    /**
     * getCustomerId
     */
    public String getCustomerId(HttpServletRequest request) {
        return swibUtils.getCustomerId(request);
    }

    /**
     * getUsername
     */
    public String getUsername(HttpServletRequest request) {
        return swibUtils.getUserName(request);
    }

    /**
     * getPassword
     */
    public String getPassword(HttpServletRequest request) {
        String jwt = SwibUtils.resolveToken(request);
        CasToken casToken = swibUtils.getCasToken(jwt);
        return (String) CacheConfiguration.getHazelcastInstance().getMap("PASSWORD").get(casToken.getTgt());
    }

    /**
     * getUserProfile
     */
    public UserProfileResponse getUserProfile(HttpServletRequest request) {
        return swibEnquiryService.getUserProfile(request);
    }

    /**
     * getUserLoginDTO
     */
    public UserLoginDTO getUserLoginDTO(HttpServletRequest request) {
        UserSessionInfo userSessionInfo = getUserSessionInfo(request);
        UserLoginDTO userLoginDTO = new UserLoginDTO();
        userLoginDTO.setCustomerId(getCustomerId(request));
        userLoginDTO.setUserName(getUsername(request));
        userLoginDTO.setPassword(getPassword(request));
        userLoginDTO.setSessionId(getSessionId(request));
        userLoginDTO.setClientId(configProperties.getDpw().getHeader().getClientId());
        userLoginDTO.setSecret(configProperties.getDpw().getHeader().getSecret());
        userLoginDTO.setTopUpUrl(configProperties.getTopup().getHeader().getUri());
        userLoginDTO.setChannel(userSessionInfo.getChannel());
        userLoginDTO.setSubChannel(userSessionInfo.getSubChannel());
        return userLoginDTO;
    }

    /**
     * create header for api t24 enquiry
     */
    public T24Header createT24Header(HttpServletRequest request) {
        UserSessionInfo userSessionInfo = getUserSessionInfo(request);
        T24Header header = new T24Header();
        header.setxIBMClientId(this.apiClientProperties.getDpw().getHeader().getClientId());
        header.setxIBMClientSecret(this.apiClientProperties.getDpw().getHeader().getSecret());
        header.setReqType(this.apiClientProperties.getCommon().getHeader().getReqType());
        header.setTrusted(this.apiClientProperties.getCommon().getHeader().getTrusted());
        header.setPriority(this.apiClientProperties.getAccount().getHeader().getPriority());
        header.setApi(this.apiClientProperties.getAccount().getHeader().getApi());
        header.setApiKey(this.apiClientProperties.getAccount().getHeader().getApiKey());
        header.setUri(this.apiClientProperties.getAccount().getHeader().getUri());
        header.setChannel(userSessionInfo.getChannel());
        header.setSubChannel(userSessionInfo.getSubChannel());
        header.setLocation(userSessionInfo.getLocation());
        header.setContext(userSessionInfo.getContext());
        header.setUserID(swibUtils.getUserName(request));
        header.setRequestAPI(swibUtils.getHostName(request));
        header.setRequestNode("01");
        return header;
    }

    /**
     * Create header request for UtilityAPI
     */
    public UtilityHeaderRequest createUtilityHeader(HttpServletRequest request) {
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        UtilityHeaderRequest header = new UtilityHeaderRequest();

        //DPW
        ConfigProperties.Dpw dpw = configProperties.getDpw();
        ConfigProperties.DpwHeader dpwHeader = dpw.getHeader();

        header.setxIBMClientId(dpwHeader.getClientId());
        header.setxIBMClientSecret(dpwHeader.getSecret());

        //utility API header
        ConfigProperties.Utility utility = configProperties.getUtility();
        ConfigProperties.APIClientHeader apiClientHeader = utility.getHeader();

        header.setReqType(apiClientHeader.getReqType());
        header.setTrusted(apiClientHeader.getTrusted());
        header.setApi(apiClientHeader.getApi());
        header.setApiKey(apiClientHeader.getApiKey());
        header.setPriority(apiClientHeader.getPriority());
        header.setUri(apiClientHeader.getUri());

        header.setChannel(userSessionInfo.getChannel());
        header.setSubChannel(userSessionInfo.getSubChannel());
        header.setLocation(userSessionInfo.getLocation());
        header.setContext(userSessionInfo.getContext());
        try {
            header.setUserID(swibUtils.getUserName(request));
        } catch (Exception e) {
            header.setUserID(apiClientHeader.getUserID());
        }
        header.setRequestAPI(swibUtils.getHostName(request));
        header.setRequestNode("01");

        return header;
    }

    /**
     * Create header request for STOAPI
     */
    public StoHeaderRequest createSTOHeader(HttpServletRequest request) {
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        StoHeaderRequest stoHeaderRequest = new StoHeaderRequest();

        //DPW
        ConfigProperties.Dpw dpw = configProperties.getDpw();
        ConfigProperties.DpwHeader dpwHeader = dpw.getHeader();

        stoHeaderRequest.setxIBMClientId(dpwHeader.getClientId());
        stoHeaderRequest.setxIBMClientSecret(dpwHeader.getSecret());

        //common header
        ConfigProperties.Common common = configProperties.getCommon();
        ConfigProperties.CommonHeader commonHeader = common.getHeader();

        stoHeaderRequest.setReqType(commonHeader.getReqType());
        stoHeaderRequest.setTrusted(commonHeader.getTrusted());
        stoHeaderRequest.setChannel(userSessionInfo.getChannel());
        stoHeaderRequest.setSubChannel(userSessionInfo.getSubChannel());

        //utility API header
        ConfigProperties.Sto sto = configProperties.getSto();
        ConfigProperties.APIClientHeader apiClientHeader = sto.getHeader();

        stoHeaderRequest.setApi(apiClientHeader.getApi());
        stoHeaderRequest.setApiKey(apiClientHeader.getApiKey());
        stoHeaderRequest.setPriority(apiClientHeader.getPriority());
        stoHeaderRequest.setUri(apiClientHeader.getUri());


        stoHeaderRequest.setLocation(userSessionInfo.getLocation());
        stoHeaderRequest.setContext(userSessionInfo.getContext());
        stoHeaderRequest.setUserID(swibUtils.getUserName(request));
        stoHeaderRequest.setRequestAPI(swibUtils.getHostName(request));
        stoHeaderRequest.setRequestNode("01");
        return stoHeaderRequest;
    }

    /**
     * Created VNPT Header Req
     */
    public VNPTHeaderReq createdBillingVNPTHeaderReq(HttpServletRequest request, String command) {
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        VNPTHeaderReq vnptHeaderReq = new VNPTHeaderReq();
        vnptHeaderReq.setCommand(command);
        vnptHeaderReq.setxIBMClientId(this.configProperties.getDpw().getHeader().getClientId());
        vnptHeaderReq.setxIBMClientSecret(this.configProperties.getDpw().getHeader().getSecret());
        vnptHeaderReq.setApi(this.configProperties.getBillingvnpt().getHeader().getApi());
        vnptHeaderReq.setApiKey(this.configProperties.getBillingvnpt().getHeader().getApiKey());
        vnptHeaderReq.setSender("VNPT");
        vnptHeaderReq.setUri(this.configProperties.getBillingvnpt().getHeader().getUri());
        vnptHeaderReq.setReq_time(System.currentTimeMillis() + "");
        vnptHeaderReq.setTraceid(System.currentTimeMillis() + "");
        vnptHeaderReq.setChannel(userSessionInfo.getChannel());
        return vnptHeaderReq;
    }

    public QRHeaderRequest createQRHeaderReq(HttpServletRequest request, UserSessionInfo userSessionInfo) {
        QRHeaderRequest headerRequest = new QRHeaderRequest();
        headerRequest.setxIBMClientId(this.configProperties.getDpw().getHeader().getClientId());
        headerRequest.setxIBMClientSecret(this.configProperties.getDpw().getHeader().getSecret());
        headerRequest.setReqType(this.configProperties.getQrTransaction().getHeader().getReqType());
        headerRequest.setApi(this.configProperties.getQrTransaction().getHeader().getApi());
        headerRequest.setApiKey(this.configProperties.getQrTransaction().getHeader().getApiKey());
        headerRequest.setPriority(this.configProperties.getQrTransaction().getHeader().getPriority());
        headerRequest.setChannel(userSessionInfo.getChannel());
        headerRequest.setSubChannel(userSessionInfo.getSubChannel());
        headerRequest.setTrusted(this.configProperties.getQrTransaction().getHeader().getTrusted());
        headerRequest.setUserID(this.configProperties.getQrTransaction().getHeader().getUserID());
        headerRequest.setRequestAPI(this.configProperties.getQrTransaction().getHeader().getRequestAPI());
        headerRequest.setRequestNode(this.configProperties.getQrTransaction().getHeader().getRequestNode());
        headerRequest.setUri(this.configProperties.getQrTransaction().getHeader().getUri());
        headerRequest.setUserID(userSessionInfo.getUsername());
        headerRequest.setLocation(userSessionInfo.getLocation());
        headerRequest.setContext(userSessionInfo.getContext());
        return headerRequest;
    }

    /**
     * checkAccountValid
     */
    public void checkAccountValid(HttpServletRequest request, String accountNumber) throws SwibApiException {
        PersonalValidator.Result rs = currentAccountValidator.validate((CurrentAccountValidator.ICurrentAccountValidator) () -> {
            List<String> accounts = new ArrayList<>();
            if (accountNumber != null) {
                accounts.add(accountNumber);
            }
            return accounts;
        }, request);
        if (!rs.isSuccess()) {
            throw new SwibApiException("Account Invalid!");
        }
    }

    /**
     * checkAccountValid
     */
    public void checkAccountValid(HttpServletRequest request, Object bodyRequest) {
        PersonalValidator.Result rs = currentAccountValidator.validate(bodyRequest, request);
        if (!rs.isSuccess()) {
            throw new SwibApiException("Account Invalid!");
        }
    }

    public void confirmOTP(HttpServletRequest request, String otp) {
        //Verify OTP
        BodyConfirmOTP bodyConfirmOTP = new BodyConfirmOTP();
        bodyConfirmOTP.setTransTime(String.valueOf(System.currentTimeMillis()));
        bodyConfirmOTP.setPassword(getPassword(request));
        bodyConfirmOTP.setTransInfo(getCustomerId(request) + "*topup-transaction");
        bodyConfirmOTP.setValidateCode(otp);
        ConfirmOTPResponse confirmOTPResponse = swibCommonService.confirmOTP(request, bodyConfirmOTP);
        if (!"00".equalsIgnoreCase(confirmOTPResponse.getResponseCode())) {
            throw new CoreApiException(confirmOTPResponse.getErrorCode(), confirmOTPResponse.getErrorMessage(), CoreApiException.PrefixErrorCode.AUTHEN);
        }
    }
}
