package vn.com.seabank.biz.web.rest;

import com.codahale.metrics.annotation.Timed;
import com.seabank.biz.custinfo.bean.UserProfileResponse;
import com.seabank.biz.sto.bean.TransBillingChangeResponse;
import com.seabank.biz.sto.bean.TransBillingResponse;
import com.swib.seanet.t24enquiry.bean.SaleAccountInfo;
import com.swib.seanet.t24enquiry.bean.T24Header;
import com.swib.seanet.t24enquiry.service.T24EnquiryApiClient;
import com.swib.seanet.topup.bean.BillingTransactionResponse;
import com.swib.seanet.topup.bean.DocumentInfo;
import com.swib.seanet.topup.dto.request.GetVNPTBillingInfoReq;
import com.swib.seanet.topup.dto.request.VNPTBillingTransactionReq;
import com.swib.seanet.topup.dto.response.GetVNPTBillingInfoRes;
import com.swib.seanet.topup.dto.response.VNPTBillingTransactionRes;
import com.swib.services.domain.VoucherBatch;
import com.swib.services.service.CustomerObserver;
import com.swib.services.service.PrePaidAccService;
import com.swib.services.service.VoucherBatchService;
import com.swib.services.service.dto.UseVoucherRequestDTO;
import com.swib.services.service.dto.VoucherUsedDTO;
import com.swib.services.utils.SwibUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import vn.com.seabank.biz.domain.ProvinceVNPT;
import vn.com.seabank.biz.domain.UserSessionInfo;
import vn.com.seabank.biz.dto.*;
import vn.com.seabank.biz.exception.CoreApiException;
import vn.com.seabank.biz.service.BillingService;
import vn.com.seabank.biz.utils.BuildHeaderUtils;

import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.List;

/**
 * Created by sonba.itsol
 * Email: sonba.itsol@seabank.com.vn
 * Date: 3/24/2020 / 03 / 2020
 * Time: 4:13 PM
 */
@RestController
@RequestMapping(value = "/api/p0605-billing")
public class BillingResource extends BuildHeaderUtils {
    private final Logger log = LoggerFactory.getLogger(BillingResource.class);

    private static final String VND = "VND";
    private static final String BILLING_TRANSFER = "BILLING_TRANSFER";

    @Inject
    private BillingService billingService;

    @Autowired
    VoucherBatchService voucherBatchService;

    @Inject
    private SwibUtils swibUtils;

    @Autowired
    PrePaidAccService prePaidAccService;

    @Inject
    private CustomerObserver customerObserver;

    private static final String CODE02 = "02";
    private static final String BANKAPI_ADMIN_VOUCHER = "BANKAPI-ADMIN-VOUCHER";
    private static final String CODE99 = "099";
    private static final String BANKAPI_NEXTGEN = "BANKAPI-NEXTGEN";


    /**
     * getAccountInfo - Lay danh sach tai khoan thanh toan theo cau hinh
     */
    @RequestMapping(value = "/get-account-info", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<Object>> getAccountInfoForBilling(HttpServletRequest request) throws Exception {
        UserProfileResponse userProfileResponse = getUserProfile(request);
        String key = userProfileResponse.getPackAge();
        ResponseObject<Object> transaction = billingService.getListAccountForBilling(request, key);
        return new ResponseEntity<>(transaction, HttpStatus.OK);
    }

    /**
     * getUserProfile - Lay thong tin khach hang
     */
    @RequestMapping(value = "/get-user-profile", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<UserProfileResponse>> getUserProfiles(HttpServletRequest httpServletRequest) {
        UserProfileResponse userProfileResponse = getUserProfile(httpServletRequest);
        return new ResponseEntity<>(ResponseObject.success(userProfileResponse), HttpStatus.OK);
    }

    /**
     * getVNPTBillingCity
     */
    @RequestMapping(value = "/get-vnpt-billing-city", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<List<ProvinceVNPT>>> getVNPTBillingCity(HttpServletRequest request) {
        return new ResponseEntity<>(billingService.getVNPTBillingCityList(request), HttpStatus.OK);
    }

    /**
     * getServiceChildren - Truy van danh sach dich vu con cua dich vu BILLING
     */
    @RequestMapping(value = "/get-service-provider", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<Object>> getServiceProvider(HttpServletRequest request) throws IOException {
        ResponseObject<Object> result = billingService.getServiceProvider(request);
        return new ResponseEntity<>(result, HttpStatus.OK);
    }

    /**
     * BillingEnquiry - Truy van hoa don
     */
    @RequestMapping(value = "/billing-Info-enquiry", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<DocumentInfo>> billingEnquiry(HttpServletRequest request, @RequestBody BillingEnquiryReq bodyRequest) {
        ResponseObject<DocumentInfo> transaction = billingService.billingEnquiry(request, bodyRequest);
        if (transaction.getData().getBill().getAmount().equalsIgnoreCase("0")) {
            throw new CoreApiException("001", "Invoice number do not owe fees.", CoreApiException.PrefixErrorCode.TOPUPBILLING);
        }
        return new ResponseEntity<>(transaction, HttpStatus.OK);
    }

    /**
     * BillingTransaction - Thanh toan hoa don
     */
    @RequestMapping(value = "/billing-transaction", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<BillingTransactionResponse>> billingTransaction(HttpServletRequest request, @RequestBody BillingTransactionReq bodyRequest) throws IOException {
        VoucherBatch validVoucher = null;
        if (!StringUtils.isEmpty(bodyRequest.getRegIssuanceName())) {
            validVoucher = voucherBatchService.getValidVoucher(request, bodyRequest.getRegIssuanceName(), BILLING_TRANSFER);
            if (validVoucher == null) {
                throw new CoreApiException(CODE02, "EVoucherCode is not exist or wrong type", BANKAPI_ADMIN_VOUCHER);
            }
        }
        //ConfirmOTP
        confirmOTP(request, bodyRequest.getOtp());
        // check topup by credit card or account
        if (StringUtils.isEmpty(bodyRequest.getTokenCard())) {
            // Kiem tra tai khoan lay sao ke phai nam
            // trong danh sach tai khoan cua nguoi dung
            checkAccountValid(request, bodyRequest.getCustomerAccount());
        } else {
            // get sale account when topup by credit card
            T24Header header = createT24Header(request);
            T24EnquiryApiClient t24EnquiryApiClient = new T24EnquiryApiClient(header);
            SaleAccountInfo saleAccountInfo = t24EnquiryApiClient.getSaleAccByTokenCard(bodyRequest.getTokenCard());
            bodyRequest.setCustomerAccount(saleAccountInfo.getAccountId());
        }
        ResponseObject<BillingTransactionResponse> transaction = billingService.billingTransaction(request, bodyRequest);
        customerObserver.onAccountChanged(swibUtils.getUserSessionInfo(request));
        try {
            if (validVoucher != null) {
                UseVoucherRequestDTO useVoucherRequestDTO = new UseVoucherRequestDTO();
                useVoucherRequestDTO.setAccountNo(bodyRequest.getCustomerAccount());
                useVoucherRequestDTO.setCreditCcy(VND);
                useVoucherRequestDTO.setDebitCcy(VND);
                useVoucherRequestDTO.setEvoucherCode(validVoucher.getEvoucherCode());
                useVoucherRequestDTO.setTransId(transaction.getData().getTransactionNO());
                useVoucherRequestDTO.setTransAmt(bodyRequest.getAmount());
                useVoucherRequestDTO.setTransTypeReq(BILLING_TRANSFER);
                voucherBatchService.usedVoucher(request, useVoucherRequestDTO);
            }
        } catch (Exception ex) {
            log.error("-----> TRANSFER VNPT BILLING ERROR:: ", ex.getMessage());
        }
        return new ResponseEntity<>(transaction, HttpStatus.OK);
    }

    /**
     * getVNPTBillingInfo
     */
    @RequestMapping(value = "/get-vnpt-billing-info", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<GetVNPTBillingInfoRes>> getVNPTBillingInfo(HttpServletRequest request, @RequestBody GetVNPTBillingInfoReq requestBody) {
        ResponseObject<GetVNPTBillingInfoRes> vnptBillingInfoResponse = billingService.getVNPTBillingInfo(request, requestBody);
        if (vnptBillingInfoResponse.getData().getBillAmount().equalsIgnoreCase("0")) {
            throw new CoreApiException("001", "Invoice number do not owe fees.", CoreApiException.PrefixErrorCode.TOPUPBILLING);
        }
        return new ResponseEntity<>(vnptBillingInfoResponse, HttpStatus.OK);
    }

    /**
     * transferVNPTBillingInfo
     */
    @RequestMapping(value = "/transfer-vnpt-billing", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<VNPTBillingTransactionRes>> transferVNPTBillingInfo(HttpServletRequest request, @RequestBody VNPTBillingTransactionReq bodyRequest) throws CoreApiException, IOException {
        log.info("-----> START TRANSFER VNPT BILLING:: ");
        VoucherBatch validVoucher = null;
        if (prePaidAccService.checkLimitRemainPrePaidCard(bodyRequest.getAccount(), Long.valueOf(bodyRequest.getBillAmount()))) {
            //check voucher information
            if (!StringUtils.isEmpty(bodyRequest.getRegIssuanceName())) {
                validVoucher = voucherBatchService.getValidVoucher(request, bodyRequest.getRegIssuanceName(), BILLING_TRANSFER);
                if (validVoucher == null) {
                    throw new CoreApiException(CODE02, "EVoucherCode is not exist or wrong type", BANKAPI_ADMIN_VOUCHER);
                }
            }
            //ConfirmOTP
            confirmOTP(request, bodyRequest.getOtp());

            // check topup by credit card or account
            if (StringUtils.isEmpty(bodyRequest.getTokenCard())) {
                // Kiem tra tai khoan lay sao ke phai nam
                // trong danh sach tai khoan cua nguoi dung
                checkAccountValid(request, bodyRequest);
            } else {
                // get sale account when topup by credit card
                T24Header header = this.createT24Header(request);
                T24EnquiryApiClient t24EnquiryApiClient = new T24EnquiryApiClient(header);
                SaleAccountInfo saleAccountInfo = t24EnquiryApiClient.getSaleAccByTokenCard(bodyRequest.getTokenCard());
                bodyRequest.setAccount(saleAccountInfo.getAccountId());
            }
            ResponseObject<VNPTBillingTransactionRes> transaction = billingService.vnptBillingTransaction(request, bodyRequest);
            customerObserver.onAccountChanged(swibUtils.getUserSessionInfo(request));
            try {
                if (validVoucher != null) {
                    UseVoucherRequestDTO useVoucherRequestDTO = new UseVoucherRequestDTO();
                    useVoucherRequestDTO.setAccountNo(bodyRequest.getAccount());
                    useVoucherRequestDTO.setCreditCcy(VND);
                    useVoucherRequestDTO.setDebitCcy(VND);
                    useVoucherRequestDTO.setEvoucherCode(validVoucher.getEvoucherCode());
                    useVoucherRequestDTO.setTransId(transaction.getData().getFt());
                    useVoucherRequestDTO.setTransAmt(bodyRequest.getBillAmount());
                    useVoucherRequestDTO.setTransTypeReq(BILLING_TRANSFER);
                    voucherBatchService.usedVoucher(request, useVoucherRequestDTO);
                }
            } catch (Exception ex) {
                log.error("-----> TRANSFER VNPT BILLING ERROR:: ", ex.getMessage());
            }

            log.info("-----> TRANSFER VNPT BILLING SUCCESS:: " + transaction.getData().getFt());
            return new ResponseEntity<>(transaction, HttpStatus.OK);
        } else {
            throw new CoreApiException(CODE99, "Prepaid card remain amount not enough", BANKAPI_NEXTGEN);
        }
    }

    /**
     * getListSTO
     */
    @RequestMapping(value = "/get-list-sto", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<Object>> getListSTO(HttpServletRequest request) throws Exception {
        ResponseObject<Object> transaction = billingService.getListSTO(request);
        return new ResponseEntity<>(transaction, HttpStatus.OK);
    }

    /**
     * transBilling
     */
    @RequestMapping(value = "/trans-billing", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<TransBillingResponse>> transBilling(HttpServletRequest request, @RequestBody TransBillingReq transBillingReq) throws Exception {
        log.info("-----> START TRANSFER BILLING:: ");
        checkAccountValid(request, transBillingReq.getDebitAccount());
        ResponseObject<TransBillingResponse> transaction = billingService.getTransBilling(request, transBillingReq.copyTo(getSessionId(request), getCustomerId(request)), false);
        log.info("-----> TRANSFER BILLING SUCCESS:: " + transaction.getData().getTransactionNO());
        return new ResponseEntity<>(transaction, HttpStatus.OK);
    }

    /**
     * transBillingVNPT
     */
    @RequestMapping(value = "/trans-billing-vnpt", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<TransBillingResponse>> transBillingVNPT(HttpServletRequest request, @RequestBody TransBillingVNPTReq transBillingVNPTReq) throws Exception {
        log.info("-----> START TRANSFER BILLING VNPT:: ");
        checkAccountValid(request, transBillingVNPTReq.getDebitAccount());
        ResponseObject<TransBillingResponse> transaction = billingService.getTransBilling(request, transBillingVNPTReq.copyTo(getSessionId(request), getCustomerId(request)), true);
        log.info("-----> TRANSFER BILLING VNPT SUCCESS:: " + transaction.getData().getTransactionNO());
        return new ResponseEntity<>(transaction, HttpStatus.OK);
    }

    /**
     * transBillingChange
     */
    @RequestMapping(value = "/trans-billing-change", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<TransBillingChangeResponse>> transBillingChange(HttpServletRequest request, @RequestBody TransBillingChangeReq transBillingChangeReq) throws Exception {
        checkAccountValid(request, transBillingChangeReq.getDebitAccount());
        ResponseObject<TransBillingChangeResponse> transaction = billingService.getBillingChange(request, transBillingChangeReq.copyTo(getSessionId(request), getCustomerId(request)), false);
        return new ResponseEntity<>(transaction, HttpStatus.OK);
    }

    /**
     * transBillingChangeVNPT
     */
    @RequestMapping(value = "/trans-billing-change-vnpt", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @Timed
    public ResponseEntity<ResponseObject<TransBillingChangeResponse>> transBillingChangeVNPT(HttpServletRequest request, @RequestBody TransBillingChangeVNPTReq transBillingChangeVNPTReq) throws Exception {
        checkAccountValid(request, transBillingChangeVNPTReq.getDebitAccount());
        ResponseObject<TransBillingChangeResponse> transaction = billingService.getBillingChange(request, transBillingChangeVNPTReq.copyTo(getSessionId(request), getCustomerId(request)), true);
        return new ResponseEntity<>(transaction, HttpStatus.OK);
    }

}
