package vn.com.seabank.biz.web.rest;

import com.codahale.metrics.annotation.Timed;
import com.seabank.biz.pfm.dto.request.InsertTransactionOnlineReq;
import com.swib.seanet.topup.bean.QRTransactionBodyRequest;
import com.swib.seanet.topup.bean.QRTransactionRequest;
import com.swib.seanet.topup.bean.QRTransactionResponse;
import com.swib.services.domain.VoucherBatch;
import com.swib.services.service.SWIBEnquiryService;
import com.swib.services.service.SwibCommonService;
import com.swib.services.service.VoucherBatchService;
import com.swib.services.service.dto.UseVoucherRequestDTO;
import com.swib.services.utils.SwibUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import vn.com.seabank.biz.domain.UserSessionInfo;
import vn.com.seabank.biz.dto.QRParameters;
import vn.com.seabank.biz.dto.ResponseObject;
import vn.com.seabank.biz.exception.CoreApiException;
import vn.com.seabank.biz.exception.NextgenApiException;
import vn.com.seabank.biz.service.QRTransactionService;
import vn.com.seabank.biz.ultil.ErrorMessage;

import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * @author anhvd-itsol on 12/9/2020
 */
@RestController
@RequestMapping(value = "/api/qr-transaction")
public class QRTransactionResource {
    private final Logger log = LoggerFactory.getLogger(QRTransactionResource.class);

    @Inject
    private QRTransactionService qrTransactionService;

    @Inject
    private SWIBEnquiryService swibEnquiryService;

    @Inject
    private SwibUtils swibUtils;

    @Inject
    VoucherBatchService voucherBatchService;

    @Inject
    SwibCommonService swibCommonService;

    private static final String VND = "VND";
    private static final String CODE02 = "02";
    private static final String QR_PAYMENT = "QR_PAYMENT";
    private static final String BANKAPI_ADMIN_VOUCHER = "BANKAPI-ADMIN-VOUCHER";

    /**
     * Giao dich qua QR code
     */
    @RequestMapping(value = "/qr-issuing-transaction", method = RequestMethod.POST)
    @Timed
    public ResponseEntity<ResponseObject<Object>> issuingQRTransaction(HttpServletRequest request, @RequestBody QRTransactionRequest bodyRequest) throws Exception {
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        VoucherBatch validVoucher = null;
        log.info("-----> START ISSUING QR TRANSACTION:: " + bodyRequest.toString());

        if (!StringUtils.isEmpty(bodyRequest.getRegIssuanceName())) {
            validVoucher = voucherBatchService.getValidVoucher(request, bodyRequest.getRegIssuanceName(), QR_PAYMENT);
            if (validVoucher == null) {
                throw new CoreApiException(CODE02, "EVoucherCode is not exist or wrong type", BANKAPI_ADMIN_VOUCHER);
            }
        }

        QRTransactionBodyRequest coreRequest = new QRTransactionBodyRequest();
        coreRequest = coreRequest.convertToCoreRequest(bodyRequest);
        QRTransactionResponse response = qrTransactionService.QRIssuingTransaction(request, coreRequest);
        try {
            if (validVoucher != null) {
                UseVoucherRequestDTO useVoucherRequestDTO = new UseVoucherRequestDTO();
                useVoucherRequestDTO.setAccountNo(bodyRequest.getPan());
                useVoucherRequestDTO.setCreditCcy(bodyRequest.getCcy());
                useVoucherRequestDTO.setDebitCcy(bodyRequest.getCcy());
                useVoucherRequestDTO.setEvoucherCode(validVoucher.getEvoucherCode());
                useVoucherRequestDTO.setTransId(response.getBankTransId());
                useVoucherRequestDTO.setTransAmt(bodyRequest.getAmount());
                useVoucherRequestDTO.setTransTypeReq(QR_PAYMENT);
                voucherBatchService.usedVoucher(request, useVoucherRequestDTO);
            }
            if (!StringUtils.isEmpty(bodyRequest.getCategoryId())) {
                InsertTransactionOnlineReq insertTransactionOnlineReq = new InsertTransactionOnlineReq();
                insertTransactionOnlineReq.setCategoryId(bodyRequest.getCategoryId());
                insertTransactionOnlineReq.setFtId(response.getBankTransId());
                insertTransactionOnlineReq.setCustomerId(swibUtils.getCustomerId(request));
                insertTransactionOnlineReq.setTransDate(dateToString(new Date()));
                swibCommonService.insertTransactionOnline(userSessionInfo, insertTransactionOnlineReq);
            }
        } catch (Exception ex) {
            log.error("-----> ISSUING QR TRANSACTION ERROR:: ", ex.getMessage());
        }

        log.info("-----> END ISSUING QR TRANSACTION.");
        return new ResponseEntity<>(ResponseObject.success(response), HttpStatus.OK);
    }

    @RequestMapping(value = "/accountInfo/{typeTransfer}", method = RequestMethod.GET)
    public ResponseEntity<ResponseObject<?>> getAccounts(HttpServletRequest request,
                                                         @PathVariable String typeTransfer) {
        log.info("-----> START GET ACCOUNTS FOR QR PAYMENT:: ");
        UserSessionInfo userSessionInfo = swibUtils.getUserSessionInfo(request);
        Map<String, Object> result = swibEnquiryService.getGroupAccountsForTransfer(userSessionInfo, typeTransfer);
        log.info("-----> END GET ACCOUNTS FOR QR PAYMENT.");
        return new ResponseEntity<>(ResponseObject.success(result), HttpStatus.OK);
    }

    private String dateToString(Date date) {
        if (date != null) {
            String pattern = "yyyyMMddHHmmss";
            DateFormat df = new SimpleDateFormat(pattern);
            return df.format(date);
        }
        return "";
    }
}
